(*********************************************************************

	Cool`AGE`  package  (Applied General Equilibrium analysis)


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1995 Thomas Cool *)
(* :Package Version: 1.04,  December 1995.*)
(* :Mathematica Version: 2.2.2 *)
(* :Keywords:    *)
(* :Limitation:  *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen,Holland.
      Please inquire about the cost of packs. See the License[] function. *)
(* :Acknowledgement: Asahi Noguchi, Silvio Levy *) 
(* : Comment: not yet experimented with ImplicitPlot, Mma/Technical Report p165*)
(* :History: This package Cool`AGE` originally started from, and now is a complete overhaul and drastic change of, a package by Asahi Noguchi 1992 supplied with the Varian Book. The package relies on the package MyFindRoot` also supplied there. The history of those two packages is: <<A GeneralEquilibrium.m was supplied in the electronic supplement of The Mathematica Journal, Volume 1, Issue 3, Winter 1991.  GeneralEquilibrium.m was written by Asahi Noguchi in August 1990, modified by Silvio Levy in March 1991, who was responsible for the general design too.>> *)

Needs["Cool`Common`"]

BeginPackage["Cool`AGE`", Join[{"MyFindRoot`", "Cool`Leontief`", "Cool`CES`", "Cool`LevelCES`", "Cool`Economic`Common`"}, $Cool]]

Options$SetModel::usage = "Useful for ResetOptions[SetModel]"

IntermediatesQ::usage = "IntermediatesQ is True or False, for when the model allows for intermediate inputs or not."

Matrix::usage = Key[AddedUsage, Matrix, "Option of Assign, required when IntermediatesQ -> True, for the matrix of intersectoral deliveries"]

SetModel::usage = "SetModel[options] loads the formula for a particular model.  Options are NumberOfSectors, NumberOfFactors, Utility, Production, IntermediatesQ, Aggregator, MaxIterations. For defaults, see Options[SetModel].\nMaxIterations is passed on to FindRoot (called by MyFindRoot).\nTypes of functions can be given (like Utility -> CES, Production -> CES) for automatic generation for the sectors and factors. The types must be mentioned in Aggregator -> {CES, ...}; and then UFunction and PFunction are called.\nAlternatively one supplies explicit specifications for Utility -> ... and Production -> ... oneself. It is advised that one uses Hold[SetFunction], in the manner as UFunction and PFunction do. See AGExample."

Assign::usage = "Assign[pars] assigns specific numerical values to the model's parameters.  One can use input {Utility -> ..., Production -> {Sector[1] -> ....}, Resources[1] -> ... (and if required Matrix -> ...)}, or just a list for the parameters. Assign can use default parameter values if these have been supplied with SetFunction."

Allocation::usage = "Allocation gives values for the production factors, under the current model and for the given parameters. A model must have been set before this function can be used: see SetModel. Input formats are:\n1) Allocation[pars_List, opts___Rule] for a list of parameters (including resources) gives the equilibrium allocations. A special option is ContractCurve -> FactorX[.,.] -> value. (You can add Sector -> {i,j}, Factor -> {k,m} or use defaults.) This gives the allocation such that other sectors are at equilibrium values, but the mentioned two sectors and two factors are solved so that the mentioned FactorX[.,.] is on the Contract Curve. Another option allows disequilibria by contraining factors to specific values, by Constraint -> {FactorX[...] -> val, ...}. Then the price equalities (vp) are preferred above the mpr's (w).\n2) Allocation[pars_List, x_List] sets the factors to the values in x, which may not be optimal. If element x[i, j] is Automatic, then the equilibrium solution for that element is taken. Per factor one element x[i, j] can best be Null so that it is computed as the remainder of the other allocations.\n3) Allocation as a Keyword: in most routines one can use the option Allocation -> y. Here y can be the output of Allocation[...], or the x-matrix, or ContractCurve -> FactorX[.,.] -> value."

AllocationTable::usage = "AllocationTable[x] prints a factor allocation assignment in TableForm"

Equilibrium::usage = "Equilibrium[pars] calls Allocation[pars] and solves key equilibrium results."

XEq::usage = "XEq[i][pars] gives the production of commodity i, for given parameters."

FactorXEq::usage = "FactorXEq[i, j][pars] gives the equilibrium allocation of factor j to sector i, for given parameter image."

wEq::usage = "wEq[i, j][pars] gives the equilibrium marginal productivity of factor j in sector i, for given parameters."

CoEq::usage = "CoEq[i][pars] gives the consumption of commodity i, for given parameters."

pEq::usage = "pEq[i][pars] gives the equilibrium price of commodity i, for given parameters. Commodity 1 is the numeraire."

vpEq::usage = "vpEq[i][pars] gives the equilibrium marginal value productivity of the factor 1 in sector i."

UEq::usage = "UEq[pars] gives the social utility for given parameters."

YEq::usage = "YEq[i][pars] gives national income, for given parameters. Commodity 1 is the numeraire."

Isoquants::usage = "Isoquants[pars_List, opts___Rule] plots 2D isoquants for options Sector -> {i, j}  and Factor -> {f, g}. Defaults are given in Options[SetModel]. If sector j === All, then isoquants for i are given (for all available resources); otherwise the isoquants for j are given (for the equilibrium allocation to sectors i + j and excluding the allocations to the other sectors)."

ContractCurve::usage = " ContractCurve[pars_List, opts__Rule] plots the 2D contract curve for factor allocations. Options are Sector -> {i, j}  and Factor -> {f, g}, with defaults given Options[SetModel]."

Isoquant::usage = "Isoquant[pars_List, opts___Rule] plots the 2D isoquant of equilibrium output of a commodity as a function of factor allocations to the sector.  Options are Sector -> {i, j}  and Factor -> {f, g}, with defaults given by Options[SetModel], and Take -> First for taking i, and Take -> Last for taking j. Factor allocations for i are read from the lower left corner in the diagram, factor allocations for j are read from the upper right corner."
  
Isocost::usage = "Isocost[pars_List, opts___Rule] plots the 2D isocost line for options Sector -> {i, j}  and Factor -> {f, g}, with defaults given by Options[SetModel]."

EdgeworthBowley::usage = "EdgeworthBowley[pars_List, opts___Rule] plots the 2D Edgeworth-Bowley diagram for options Sector -> {i, j} and Factor -> {f, g}. Defaults are given by Options[SetModel]."

PPC::usage = "PPC[pars_List, opts___Rule] gives the production possibility curve. See CPCDiagram."

PPCBudget::usage = "CPCBudget[pars, opts] plots the equilibrium production budget line for two sectors assuming the others to be constant."

IndifferenceCurve::usage = "IndifferenceCurve[pars, opts] plots the equilibrium indifference contour for two sectors assuming the others to be constant."

CPCBudget::usage = "CPCBudget[pars, opts] plots the equilibrium consumption budget line for two sectors assuming the others to be constant."

CPCDiagram::usage = "CPCDiagram[pars_List, opts___Rule] combines the item plots of the Consumption Possibilities Curve, the Indifference contour, and the budget line, for two sectors assuming the other sectors to be constant. The sectors are selected with the Sector -> {i, j} option. If IntermediatesQ -> True, then the Production Possibilities Curve can be included by giving CPCDiagram -> All. Use PlotPoints to get smoother lines. Defaults are given in Options[SetModel]. Use Results[CPCDiagram] for separate item plots. Numerical data per item can be found in Results[item]."

CPC::usage = "NPPC[pars_List, opts___Rule] gives the consumption possibility contour. See CPCDiagram."

Results$AGEmodel::usage = "Useful for a ResetResults[AGEmodel]"

AGEmodel::usage = "Keyword for storage in Results[AGEmodel]"

AGExample::usage = "SetModel @@ AGExample gives a 2x2 example of only unknown resource parameters. An AGExpars is created that may supply such data. For example evaluate:\n   Path[Array[CoEq,2],Table[AGExpars[i],{i,1,91,10}]]\nCompliments to Asahi Noguchi & Silvio Levy."

PFunction::usage = "PFunction[type, i_, m_] gives the production function for Sector i with m factors of production. Type must be a member of the Aggregator option of SetModel. For type = LevelCES then m is the structure.  Note: a type function best uses Hold[SetFunction][type, ...]."

UFunction::usage = "UFunction[type, n_] gives the social welfare utility function with n sectors or consumption goods. Type must be a member of the Aggregator option of SetModel. For type = LevelCES then m is the structure.  Note: a type function best uses Hold[SetFunction][type, ...]."

$Utility::usage = "Contains the results of the Utility option in SetModel.\nCommonly it gives the result of SetFunction. Internally there is a replacement of Utility[i] with Consumption[i] and then a ReleaseHold. The reason for this is that Consumption[i] will need to get a value during computations."

$Production::usage = "Contains the results of the Production option in SetModel (commonly the result of SetFunction after ReleaseHold)"

$Coefficients::usage = "Contains the names of all coefficients in the model"

StartValues::usage = Key[AddedUsage, StartValues, "StartValues[opts] gives the starting values to find the optimal allocation. Settings for opts are:\nStartValues -> {f, number} to start with f[Resources, num]. For example {Plus, -.0001} or {Divide, NumberOfSectors}. The latter is default in Options[SetModel].\nStartValues -> {FactorX[1, 1] -> ...} with actual settings that e.g. come from another exercise."]

Path::usage = "Path gives the development of variables subject to different parameter values.\nPath[{pars1, ..., parsn}] determines the allocations (using the former result as startingvalue for the new).\nPath[y, {pars1, ..., parsn}] gives the plot for y. That y can be a single item or a list, like {z[1], ..., z[m]}. The requirement is that each single y is defined like UEq and YEq, thus, y[parsi] must render a value. Those values are in Results[Path]. ListPlot is used, so small sets of parameters gives kinked output."

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
