(**************************************************************** 
 
	Cool`Chi2` package 
 
 
 *****************************************************************) 
 
 
 
(* :Author: Thomas Cool *) 
(* :Copyright: Copyright 1995 Thomas Cool *) 
(* :Package Version: 2.0 Thomas Cool, June 1997.*) 
(* :Mathematica Version: 3.0.0 *) 
(* :Keywords: statistics, chisquare *) 
(* :Limitation: *) 
(* :License: use of this software file requires a license, available 
  by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
  Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland. 
  Please inquire about the cost of file packs. See the License[] function.
 *) 
 
Needs["Cool`Common`"] 
 
BeginPackage["Cool`Chi2`", Join[{"Cool`Statistics`Common`", 
"Statistics`NormalDistribution`", 
(* Needed by S`ND anyway: *) "Statistics`Common`DistributionsCommon`", 
"Statistics`DescriptiveStatistics`", 
(*and for name conflicts: *) "Statistics`HypothesisTests`"}, $Cool]] 
 
 
$KBChi2 = Key[Begin, "Cool`Statistics`Common`", $Context] 
 
AddedUsage[{All, Out, MessagesQ, Dimensions, Show, Infinity, Indeterminate, 
ComplexInfinity, SameQ, RealQ, Partition}] 

Cool`Statistics`Common`DegreesOfFreedom::usage = Key[AddedUsage, DegreesOfFreedom, "The option occurs in output of various procedures, with values depending upon assumptions. The option may be set in Options[Chi2] for input when one has a fixed problem at hand."] 
 
FIDLLRStatistic::usage = "Full Independence Log Likelihood Ratio Statistic. This statistic assumes that all dimensions are independently distributed, and it evades the computation of inner cell ratios.\n1) FIDLLRStatistic[x_List, 
r_List:{}] where x can be a crosstable or a list of rules (the routine checks on the occurrence of a rule in the first item), or\n2) FIDLLRStatistic[
CrossTable -> x, r_List:{}]], or\n3) FIDLLRStatistic[Rule -> x, r_List:{}].\nUseful input rules must mention BorderSums, CrossTable, Dimensions, and NumberOfObservations. Also, r is a list of rules that will be used with priority, thereby possibly preventing recomputations on the original table.\n
Output is {Test -> FIDLLRStatistic, TestStatistic -> test statistic}." 
 
BorderSums::usage = "The border frequency values per category per dimension, or PrMarginal * NumberOfObservations." 
 
CrossTable::usage = "1) basically an option keyword giving a list,\n2) CrossTable[..,..,..] may give a cell within the cross table." 
 
Options$Chi2::usage = "Contains the Options[Chi2] at load time of the package. It can be used to undo changes, by ResetOptions[Chi2]" 
 
Options$Chi2 = Sort[{	SignificanceLevel -> .05, 
			DegreesOfFreedom -> Null, 
			NumberOfObservations -> Null, 
			Test -> Null, 
			TestStatistic -> Null, 
			Show -> All, 
			Infinity -> 0., Indeterminate -> 0., 
			ComplexInfinity -> 0., 
			SameQ -> True, 
			MessagesQ -> False, 
			RealQ -> True, 
			Partition -> All}] 
 
Options[Chi2] = Options$Chi2 
 
Chi2::usage = "(1) Either Chi2[x_Real, df_Integer]\n(2) or Chi2[x, DegreesOfFreedom -> df]\n(3) or Chi2[x] with default df defined by Options[Chi2].\nNote: default option is Null, but may be set for a fixed problem.\nThey all give the cumulated density\nCDF[ChiSquareDistribution[df], x]." 
 
Chi2CriticalValue::usage = "Chi2CriticalValue[y___] computes the Chi2 critical value for the pair {df, signif}, i.e the value of x with cumulated probability up to (1 - signif).\n y___ is either the sequence [empty], [df], [df, signif], or option rules. Defaults are defined in Options[Chi2]." 
 
Chi2PValue::usage = "(1) Either Chi2PValue[x_Real, df_Integer]\n(2) or Chi2PValue[x,  DegreesOfFreedom -> df]\n(3) or Chi2PValue[x] with default df defined by Options[Chi2].\nNote: default option df -> Null, but may be set for a fixed problem. \nThey all give a rule with the cumulated probability beyond x, thus Chi2PValue -> 1 - Chi2[x,df]. \nNote: the ChiSquarePValue in the Mathematica statistical packages is incorrect." 
 
InverseChi2::usage = "InverseChi2[df_, p_] gives the inverse, i.e. x = Chi2[-1][df, p] for given df and cumulated probability p up to x. No options and defaults." 
 
LogLikelihoodRatio::usage = "LogLikelihoodRatio[o_, t_, r___Rule] computes theTestStatistic:\n - 2 Log[LikelihoodRatio[o,t]] where o are the observations (frequencies i.e. levels) and t the theoretical probabilities (relative frequencies).\nInputs o and t can be multidimensional as long as they have thesame shape.\nThe DegreesOfFreedom in output is just the length of Flatten[t] minus 1. See Test for options control. Note: the test cells can be found in LogLikelihoodRatio[Table]." 
 
MLEstimate::usage = "Option in output, giving the maximum likelihood estimate against which was tested." 
 
Pearson::usage = "Pearson[o, t, r___Rule] computes the Pearson TestStatistic: \n Sum[(o - t)^2/t] where o are the observations and t the theoretical values. Inputs o and t may be multidimensional as long as they have the same shape. The DegreesOfFreedom in output is just the length of the number of observations minus 1. If Show -> All||True then the TensorForms are printed; this is default in Options[Chi2]. See Test for options control. Note: the test cells can be found in Pearson[Table]." 
 
PrEst::usage = "1) PrEst[ID, x_List, opts___] computes the theoretical probabilities under assumed independence as stated in Partition -> y_List. Default, in Options[Chi2], is Partition -> All, meaning that all dimensions 
are assumed to be independent. The routine uses BorderTotals(Q).\n2) PrEst[ID, Table] contains the result too. When Show -> None, then the sometimes rather long list is suppressed in output, which of course causes that this output may be less relevant as a Rule." 
 
Cool`Statistics`Common`ProbabilityMatrix::usage = Key[AddedUsage, ProbabilityMatrix, "Here the probabilities in the whole tensor must add up to 1."] 
 
Cool`Statistics`Common`Test::usage = Key[AddedUsage, Test, "\n(1) Test[ID, x_List, opts___] is the preferred procedure: it computes the theoretical distribution under independence, determines the (minus 2) LogLikelihoodRatio, and applies the Chi2 test. Defaults are in Options[Chi2]. The major controls are Partition and SignificanceLevel, while Show and RealQ control output formats. Note that algebraic solutions differ per partition, and that the computation of ratios per cell then is most general. Thus, instead of FIDLLRStatistic, Test uses (2).\n(2) Test[Chi2, test_List, opts___] applies the Chi2 test on the TestStatistic and SignificanceLevel parameter data in (with declining priority:) opts, test and Options[Chi2]. The used PValue test is quicker than the inverse via the critical value.\n(3) Test[Chi2, Pearson, x_List, opts___] for Pearson (instead of LLR with Test).\n(4) As in Cool`Statistics`Common`: option in output, showing the kind of test used.\n(5) Test[Chi2CriticalValue, testStatistic_, f_Integer:Automatic, sg_:Automatic, opts___Rule] applies the onesided Chi2 test on the test statistic for the pair {df, sg}, using the inverse calculation of the critical value for the test statistic. Here, {df, sg} input is either the sequence [empty], [df], [df, sg], or option rules. Defaults are defined in Options[Chi2].\n(6) Test[Show, o_, t_, cs_, headlabel_String] is a subroutine that prints the observed and theoretical input, and the cell statistics cs[o,t] in TensorForm (transposed for one dimension).\n(7) Subroutine Test[Out, ...] for the output rule of above. \n(8) Options with defaults:\n(8a) Control of 0 entries. Defaults set Pearson and LLR cells to zero when 0 is encountered. Note that x/0 = ComplexInfinity and Log[ComplexInfinity] = Infinity; that 0/0 = Indeterminate and Log[Indeterminate] = Indeterminate; while Log[0] = - Infinity.\n(8b) If MessagesQ -> False, then 0-messages are suppressed.\n(8c) If SameQ -> True then the input is tested on consistency.\n(8d) If RealQ -> True, default, then Rational's are set to Real's."] 
 
SumLog::usage = "SumLog[x_] adds the log values of x, which may be any list. Options[Chi2] gives Infinity -> 0 for Log[x[i] = 0], under the assumption that this concerns observations that can be neglected." 
 
xLogX::usage = 
	"xLogX[x_] := \nSum[If[x === 0. || x === 0, 0, N[x Log[x]]]]" 

xPowerX::usage = 
	"xPowerX[x_] := \nProduct[If[x === 0. || x === 0, 1, N[x^x]]]" 
 
xLogP::usage = 
	"xLogP[p_, x_] = \nSum[If[p === 0. || p === 0, 0, N[x Log[p]]]]"




Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
