(*********************************************************************

	Cool`Context`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1993/97 Thomas Cool *)
(* :Package Version: 2.01 Thomas Cool, July 1997.*)
(* :Mathematica Version: 3.0 *)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)
(* :Keywords:   Context, Miscellaneous, Variations on Themes *)
(* :Limitation: *)
(* :Comment: desirable are Set and Clear procedures that allow to target variables by Strings of their names. Strings namely can be manipulated, while variables are evaluated (except by current Set and Clear - that however do not allow manipulation).
*)



BeginPackage["Cool`Context`", {"Cool`Common`"}]

AddedUsage::usage = "AddedUsage[context] is a list of Keywords for which the usage is extended. Context is where the extension took place. The original Keyword::usage statements may have been extended by Key[AddedUsage, ... ]. This however need not happen, as usage can be explained by the innovating routines.\nAddedUsage[] applies for the whole $ContextPath.\nAddedUsage[x_List] puts the elements of x into the AddedUsage[$Context]."

AppendToInContext::usage = "AppendToInContext[context_String, var_String, value_String]\n appends value to var in the context"

ArgumentQ::usage = "Option for PutIntoContexts, standard True, holding that the argument is also contexted"

BeginContext::usage = "BeginContext[x_String] is similar to Begin[x] and also Prepend[$ContextPath, x]. Additional features are:\nBackApostrophe '`' may be appended to x\nBeginContext[N] gives the number of additions to $ContextPath\nBeginContext[x_NumberQ] gives the history\nBeginContext[Needs, x_String] applies Needs[x], while updating BeginContext[N] with BeginContext[Plus]"

CtE::usage = "CtE[context_String,var_String] concatenate strings & ToExpression"

ClearInContext::usage = "ClearInContext[c_String, x_String] clears variable in context using Unset.\nClearInContext[c_String, x_List], first maps a concatenation and then applies Clear.\nClearInContext[c_List, x_String] maps ClearInContext to the list of contexts, again using Unset.\nWarning: it is not always sure that the procedures work.\nHighly desirable are Set and Clear options that allow to target variables by Strings of their names. Strings namely can be manipulated, while variables are evaluated (except by current Set and Clear - that do not allow manipulation). E.g. when name = \"var\" then Clear[name] now clears name and not var."

Options[Contents] = {NeedsQ -> True}

Contents::usage = "Contents[x_context|ListOfContexts, r___Rule] \n shows the names in the (list of) contexts. \nThe default is $ContextPath minus the System context. When NeedsQ -> True (default), Needs is applied, i.e. a context is loaded from package if it does not exist yet. Use $Path for locations. If you want to restore removed variables of the context, then use Load."

Cure::usage = "Cure[c_String] adds a ` if c doesn't have one; 'Cure' comes from 'ContextSure'"

DeCure::usage = "DeCure[c_String] drops a ` if c does have one"

DataPackage::usage = "DataPackage[dir_String,ps_String] gives dir<>ps<>.m"

Discard::usage = "Discard[context_String] \n goes to Global`, clears and removes the mentioned context. NB. if ` lacks then this is appended"

KeepVariables::usage = "KeepVariables[context_String, variableNames_List, keep_List] \n removes variables from a context` by mentioning what is to be kept"

Key::usage = "Allows one to reset definitions e.g. of keywords in a common package.\n1) Key[Install, context_String,  x_List] installs Key[context] as a list of Strings for the elements in x, and sets its counter Key[context, N] to zero.\n2) Basic elements are Key[context, name, counter], giving the definition at counter as a String.\n3) Key[Set, context] increases the counter and sets the elements at the current definitions.\n4) Key[Take, context, int_Integer:0, clrq_:True] submits these definitions to ToExpression, which executes them. If int is 0 then the current counter is taken, a negative value is regarded as steps back, and otherwise one has to provide a nonnegative integer. Default clrq = True, meaning that the variables are first Cleared.\n5) Key[Begin, c, c2] records, with {Key[c, N], Date[]}, when context 2 was started and what counter applies.\n6) Note that Definition does not render usage statements. Therefor, we only deal with addition and not reset:\n   Key[AddedUsage, item_Symbol, txt_String, unknownProtection_:False] is used as the rhs of item::usage = rhs. It adds txt to the current usage statement if $Context does not yet occur in that usage statement, while item is also included in the AddedUsage[$Context]. One can enter True if one wishes to have warning messages on protected symbols." 

Load::usage ="Load[file_String, c_:C:wnmath22] gets a c`file.m and shows the contents of the first context of $ContextPath"

NeedsQ::usage = "Option of Contents. If NeedsQ -> True (default), then Needs is applied to the contexts, so that nonexisting contexts are gotten from packages. For file locations you may set the $Path"

MessagesQ::usage = "If MessagesQ -> False, then all Messages, notably of shadowing, are repressed. Only True or False are allowed. See ShadowHull."

Pralse::usage = "Pralse[x_String]: Message x, and Throw[False]\n Pralse[x_String, Print] prints instead of a message, e.g. when there is a ShadowHull"

Options[PutIntoContexts] = {ArgumentQ -> True, MessagesQ -> False}

PutIntoContexts::usage = "PutIntoContexts[y_String, f_String, x_String, c_ListOfStrings (, Options)] \n maps y=f[x] into the contexts mentioned in c, and by default x is in context, and there are no shadowing messages"

Query::usage = "Query[q_String, (C,) x___(Strings||ListsOfStrings)_inanyorder] makes the outer product of the strings||Lists, concatenates, prepends q, appends * and applies ToExpression. By taking q = \"?\" or \"??\" one can look for variables in contexts. Mentioning C includes the $ContextPath minus System`."

RemoveVariables::usage = "RemoveVariables[context_String, variableNames_List] \n removes variables from a context` by mentioning what is to remove"

ResetAll::usage = "ResetAll calls CleanSlate, or if necessary loads it"

ResetStore::usage = "Options[ResetStore] is a list of options that in case of some special packages like FilledPlot.m appears to be necessary to store options for ResetAll"

ResetGraphics::usage = "ResetGraphics calls ResetStore to restore the Options[Graphics] to the value before the loading of CleanSlate"

Scoring::usage = "Scoring[context_String, scorevar_String, func_, score_:\"Null\"]\n gives score to the scorevar in the positions determined by the function on the context"

ShadowHull::usage = "ShadowHull[{head, rule}, procedure] may suppress all messages within the procedure, and notably the name shadowing when the procedure applies ToExpression with contexts. Normally head is the name of the calling routine. ShadowHull requires that Options[head] or rule contain MessagesQ -> False (suppress) or True. Note that the rule can be used in procedure again. Example: f[y_, r___Rule] := ShadowHull[{f,r},  Module[{x}, body[y,r,x] ]]"

ShowPrivate::usage = "ShowPrivate[x_, item_:\"??\", priv_:\"Private`\"] for x that can be of Symbol or Literal[expr]. Checks whether there exists a private context, enters it, evaluates, and leaves again. In this manner Private context heads are not shown in output. For Head[x] = Symbol, default item is \"??\", but it might also be \"?\" and priv might be \"\". For x = Literal[expr] output is Print[expr]." 

Begin["`Private`"]

AddedUsage[x_] := {}

AddedUsage[] := Union[Flatten[ Map[AddedUsage, $ContextPath]]]

AddedUsage[x_List] := 

	(AddedUsage[Evaluate[$Context]] = Union[AddedUsage[$Context], x])

AppendToInContext[context_String, var_String, value_String] :=

	ToExpression["AppendTo[" <> Cure[context] <> var <> "," <> value <>"]"]


BeginContext[N] = 0
BeginContext[0] = $ContextPath

BeginContext[x_String] := 

	(If[!MemberQ[$ContextPath, Cure[x]], 
		(PrependTo[$ContextPath, Cure[x]]; BeginContext[Plus]) ]; 
	Begin[Cure[x]])

BeginContext[Plus] := (BeginContext[N] = BeginContext[N] + 1;
				BeginContext[BeginContext[N]] = $ContextPath)

BeginContext[Needs, x_String] := (Needs[x]; BeginContext[Plus])

CtE[context_String,var_String] :=   ToExpression[Cure[context] <>var]

ClearInContext[c_String, x_String] :=  ToExpression[ Cure[c] <> x <> " =. "]

ClearInContext[c_String, x_List] := Apply[Clear, Map[(Cure[c] <> #)&, x]]

ClearInContext[clist_List, x_String] :=  Map[ ClearInContext[#,x]&,  clist]

Contents[ca_:Automatic, r___Rule ] :=

	Module[{c, b = False (*bingo*)},
	c = If[ca === Automatic, DeleteCases[ $ContextPath, "System`"],
	      (*else*) If[ListQ[ca], Map[Cure, ca], b = True;{Cure[ca]}]];
	If[NeedsQ /. {r} /. Options[Contents], Map[Needs, c]];
	(* If bingo, it may be that the Path name differs from the Context *)
	If[b, If[!MemberQ[$ContextPath, Cure[ca]], c = {First[$ContextPath]}]];
	Map[Query["?",#]&,  c];
	] (* end Contents *)



Cure[c_String] := If[c === "", c, If[StringTake[c, -1] =!= "`", c<>"`", c]]	

DeCure[c_String] := If[StringTake[c,-1] =!= "`", c, StringDrop[c,-1]]

DataPackage[dir_String, ps_String] :=
			StringReplace[(dir <> DeCure[ps] <> ".m"), "`"-> "\\"]

Discard[context_String] :=  Module[{ctx},

		$Context = "Global`";  
		ctx = Cure[context] <> "*";
		Clear[Evaluate[ctx]]; Remove[Evaluate[ctx]];

		] (* end module Discard *)


KeepVariables[context_String, names_List, keep_List] :=

	RemoveVariables[context,  Complement[names, keep]]


Key[Install, c_String,  x_List] := 

	(Key[c, N] = 0; Key[c] = Map[ToString, x])

Key[Set, c_String] := (Key[c, N]++;
		Map[(Key[c, #, Key[c, N]] =
		DefinitionToString[ToExpression[#]])&, Key[c]])

Key[Take, c_String, int_Integer:0, clrq_:True] := 

	Module[{h, i, r}, 
			If[clrq, h = Map[(c <> #)&,  Key[c]]; Apply[Clear, h] ];
			i = If[Positive[int], int, Key[c, N]+int];
			r = Map[Key[c, #, i]&, Key[c]];
			ToExpression[r]]

Key[Begin, c_, c2_] := Key[Begin, c, c2] = (Key[Set, c]; {Key[c, N], Date[]})

Key::adup = "AddedUsage warning on Protected Symbol `1` "

Key[AddedUsage, item_Symbol, txt_String, unknown_:False] := 

	Module[{name, val, s, upd = True, au = AddedUsage[$Context]}, 
	If[MemberQ[Attributes[Evaluate[item]], Protected] && unknown, 
		Message[Key::adup, item] (* no further action needed*)];
	s = "\n        *** AddedUsage by " <> $Context <> ": *** \n";
	name = Context[Evaluate[item]] <> ToString[item] <>  "::usage"; 
	val = ToExpression[name];
	(* in case the package is loaded several times: *)
	s = If[StringPosition[val, s] === {}, val <> s <> txt, upd = False; val];
	If[upd, AddedUsage[Evaluate[$Context]] = Union[au, {item}]];
	s] (*end module*)


Load[file_String, c_: "C:\\wnmath22\\"] :=

		Module[{f},  f = DeCure[file] <> ".m";
				f = StringReplace[ f, "`" -> "\\\\"];
				Get[f];
				Contents[First[$ContextPath]]
			] (* end module Load *)


Pralse::throw = "Pralse exit: `1`";

Pralse[x_String] := If[ToString[$Messages] =!= "$Messages", 
				Pralse[x, Message], Pralse[x, Print]]

Pralse[x_String, Message] := (Message[Pralse::throw, x]; Throw[False])

Pralse[x_String, Print] := (Print[x]; Throw[False])


PutIntoContexts[y_String, f_String, x_String, c_List, opts___Rule] := 

		ShadowHull[{PutIntoContexts, opts},
		Module[{a, b, xIn},
		xIn = ArgumentQ /. {opts} /. Options[PutIntoContexts];
		If[xIn, 

			a = y<>"="<>f<>"[";
			b = x<>"]";
			Map[ToExpression[Cure[#]<>a<>Cure[#]<>b]&, c], (*else*)

			a = y<>"="<>f<>"["<>x<>"]";
			Map[ToExpression[Cure[#]<>a]&, c]
			];
		] (*end ShadowHull *) ] (* end Module *)

Query[q_String, txt__String] := Query[q][txt]

Query[q_String, C (*contexts*), txt__String] := 

	Query[q, DeleteCases[ $ContextPath, "System`"],  txt]

Query[q_String][txt___String] := 
		(Print["\n"<>q<>txt<>"*"];ToExpression[q <> txt <> "*"])

Query[q_String][{x___List}] := Outer[Query[q], x]

Query[q_String, x__] := Query[q][Map[If[StringQ[#], {#}, #]&, {x}]]


RemoveVariables[context_String, names_List] :=

	Do[ ToExpression["Remove[" <> Cure[context]  <> names[[i]] <> "]"],
	
	{i,1,Length[names]}]

ResetStore[] = "Works with (Set)Options[ResetStore] only"

Options[ResetStore] = {Graphics -> {}}

ResetGraphics := 	
	Module[{m}, m = Graphics /. Options[ResetStore];
		If[m =!= Graphics,
		Unprotect[Graphics];
		Options[Graphics] = m;
		Protect[Graphics]]]

ResetAll := 
	(*the FilledPlot package adjusts options of Graphics
	in such a manner that CleanSlate` removes them, again
	such that Graphics no longer work.
	Therefor:*)
	If[!MemberQ[$ContextPath, "CleanSlate`"],
	  If[{}=== StringPosition[ToString[Options[Graphics]], "Removed"],
		(*store current graphics options*)
		SetOptions[ResetStore, Graphics -> Options[Graphics]]];
		Needs["CleanSlate`"]; BeginContext[Plus],
     (*else*)
		ToExpression["CleanSlate[]"];
   	If[{}=!= StringPosition[ToString[Options[Graphics]], "Removed"],
		ResetGraphics]
	]; (*end ResetAll*)


Scoring[context_String, scorevar_String, func_, score_:"Null"] :=

	Module[{a,b}, 
	
	a = Cure[context] <> scorevar;
	b = ToString[func[context]];

	ToExpression[a <>"=ReplacePart[" <>a <> ","
			 <> score <> "," <> b <> "]"]

	] (* end module Scoring *)

ShadowHull::messq = "MessagesQ not True or False, but: `1`";

ShadowHull[{h_, opts___Rule}, x_] :=

		Module[{mq, $M, result},
		mq = MessagesQ /. Flatten[{opts}] /. Options[h];
		If[ !MemberQ[{True, False}, mq],
			Message[ShadowHull::messq, mq],
			(*else*)		
			If[!mq, $M = $Messages; $Messages =.];
			result = x;
			If[!mq, $Messages = $M];
			result
			] (*end MemberQ*)
		] (* end module *)

ShowPrivate[x_, item_:"??", priv_:"Private`"] :=
	Module[{y, h}, 
		h = Head[x];
		If[MemberQ[{Symbol, Literal}, h], y = Context[x],
			Print["Input is not Symbol or Literal"];Return[]];
		If[StringPosition[y, "Private`"] =!= {}, 
			Print["warning: already Private"];
			If[priv =!= "",Print["Perhaps use third parameter = \"\""]]];
		y = y <> priv; (*priv might be "" *)
		If[MemberQ[Contexts[], y],
			Print[y];
			Begin[y];
			If[ h === Symbol, ToExpression[item <> ToString[x]],
			(*else Literal*) Print[ First[x] ]];
			End[];, 
		(*else*) Print["No additional " <> priv <> " context."];
			If[ h === Symbol, ToExpression[item <> ToString[x]],
			(*else Literal*) Print[ First[x] ]];
		] (*end if *)
	] (*end module*)

End[]

AddedUsage[{C, Begin, Install, N, Needs, Message, Plus, Print, Set, Take}];

EndPackage[]
