(*********************************************************************

	Cool`Decision`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1995 Thomas Cool *)
(* :Package Version: 1.08 Thomas Cool, July 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation:
*)
(* :License: use of this software requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function. *)

Needs["Cool`Common`"]

BeginPackage["Cool`Decision`", Join[{
"Statistics`DescriptiveStatistics`",
"DiscreteMath`ComputationalGeometry`",
 "Cool`Statistics`Common`"}, $Cool]]


$KBDecision = Key[Begin, "Cool`Statistics`Common`", $Context]

AddedUsage[{Act, Est, Dec, State, All, Table, Show, Hull, Plot, Rule}]

Actions::usage = "Array of Act"

AnyRule::usage = "useful for demonstrations"

Assign::usage = "Assign[item, rule] assigns a numeric value to item" 

BestInRow::usage = "BestInRow[x_List] gives the best value for each state of nature, seen from the viewpoint of the player against nature. Used in EVPerfectInformation[] and Regret[]"

ByNegativeInverseSlope::usage = "Subroutine of Value2By2[], that uses the property that the minimax rule has t perpendicular to the line in the risk plot"

Decisions::usage = "Array of Dec"

DefDec::usage = "DefDec[rule] defines the various decision rules"

DirectRisk::usage = "DirectRisk[t_, x_] := {t,  1 - t} . x  generates the direct risk for apriori t = Pr[ State[1] ] when NumberOfStates = 2"

Draw::usage ="When NumberOfStates = 2, some plots are possible of loss or risk array x:\nDraw[Hull,x] plots the convex hull in the {Risk[1], Risk[2]} dimensions\nDraw[Plot,x] plots the risk (expected loss) of the various decisions subject to the probability distribution {t, 1-t}, for t in [0,1]\nDraw[PowerOfTheTest, risk, varsymb_Symbol, True|False] makes a similar 3D plot for risk[varsymb], for varsymb in [0,1]. Often varsymb = alpha = the failure rate of estimation (normally giving symmetric results around alpha = .5). If True (default): the minimum risk of all decisions is given; if False, all decisions are plotted separately.\nDraw[Value2By2, losstable] makes a 3D plot of the value of the game for distributions t and p in [0,1]"

Estimates::usage = "Array of Est"

EVPerfectInformation::usage = "EVPerfectInformation[{r__Rule}, pr_List] or EVPerfectInformation[payofftable, pr_List] gives the expected value of perfect information for the player against nature, with pr the list of probabilities assigned to the states of nature.\nNote: If the player had perfect information about the state of nature (row), he could select the colum with the minimal value. Now one can only weigh these minima with the probabilities. The extend that this result goes above the normal minimal expected loss is the value of perfect information"

GameSplit::usage = "GameSplit[game] splits a Game Theory game into two parts; without changing its meanings; see GameToPayOffRules"

GameToPayOffRules::usage = "GameToPayOffRules[game] allows an interface with the nash`nash` package (Varian's book). This procedure 'splits' the pay-off elements of the Game representation into the single pay-off item of this package. Each player is considered a decision maker playing against nature; therefor all values must be multiplied by (-1) (giving nature's revenue and decision maker's loss), and the first player's matrix must be transposed."

LookAt::usage = "LookAt[item, rule_] shows properties of the mentioned item.\nYou find the possible items by Evaluating ??LookAt"

Loss::usage = "Individual items in the payoff table. \nStructural or numeric values are allowed"

MinimalExpectedLoss::usage = "MinimalExpectedLoss[{r___Rule}, p_List] gives the minimal expected loss for the player against nature in the decision problem {r} that contains a PayOff -> ... statement, using the probability measure p for the states of nature\nMinimalExpectedLoss[payofftable, p_List] works directly on that table"

OddOrEven::usage = "This is an example of a rule that defines a decision situation.\nTwo players (nature and the statistician) may show 1 or 2 fingers.\nThe pay-off is the sum; odd values go to nature, even values to the statistician"

PayOff::usage = "Parameter 'PayOff -> NumericMatrix' occurs in a rule\nThe decision problem is regarded as a game of the Statistician against Nature. The game is represented by a PayOff table, where Nature controls the rows (horizontal player) and the Statistician controls the columns (vertical player). A positive entry is a revenue for Nature and a loss for the Statistician. PayOff refers to the whole matrix, and Loss to the elements therein"

PayOffTable::usage = "Structural or numeric values are allowed"

PayOffToRule::usage = "PayOffToRule[matrix] translates a payoff matrix of a decision problem into a rule that can be used by the other routines"

PayOffRuleToGame::usage = "PayOffRuleToGame[rule] allows an interface with the nash`nash` package (Varian's book). This procedure translates each single pay-off item X of this package into the pay-off elements {X, -X} of the Game representation"

POT::usage = "POT[rule] subfunction for Assign & LookAt"

Cool`Statistics`Common`Pr::usage = Key[AddedUsage, Pr, "An array Pr[state -> estimate of the state]\nStructural or numeric values are allowed.\nReset[Load] sets Pr to its load time value."]

Cool`Statistics`Common`ProbabilityMatrix::usage =  Key[AddedUsage, ProbabilityMatrix, "Option for ProbMat or Assign[Pr, rule].\nIn these applications the numeric values per row must add up to one."]

ProbMat::usage = "ProbMat[rule] constructs the probability matrix Pr[state->est]\nSince states of nature are estimated, this matrix is always square"

Regret::usage = "Regret[payoff table or decision problem list of rules] substracts the minimal value in the row from the whole row. A positive value is a relative loss to the vertical player and a relative revenue to the horizontal player (nature). I.e. relative compared to what would have been the best choice of the vertical player had he known the choice of the horizontal player (the state of nature)"

Reset::usage = "Reset[] sets decision variables to structural form\nReset[Load] sets them to load time values"

RiskTable::usage =  "Structural or numeric values are allowed"

Risk::usage = "individual entries in the risk table"

RiskMat::usage = "RiskMat[rule] determines the risk matrix"

RT::usage = "RT[rule] subfunction for LookAt[...]"

States::usage = "Array of State, i.e. states of nature (to be estimated)"

ToLoss::usage = "A replacement rule to determine the risk of a decision step"

Value::usage = Key[AddedUsage, Value, "Value[table, t, p] or Value[table, {t,p}] gives the value of the game\nGenerally value = t . table . p\nHere value = PML[t].table.PML[p]"]

Value2By2::usage = "Value2By2[table] determines the value of the 2 by 2 case. Output = {valmax, {'t'-> tmax, 'p' -> pmax}}. Note: not all points may be generated."

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
