(*********************************************************************

	Cool`DifData`    

 *********************************************************************)
(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1993/96 Thomas Cool *)
(* :Package Version: 3.0 Thomas Cool, June 1997*)
(* :Mathematica Version: 3.0 *)
(* :Keywords:     data, Dif, Excel *)
(* :Limitation:	Twodimensional. Not all Dif combinations have been tested.*)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)
(*********************************************************************

	->  external data input using Data Interchange Format
	->  output of data files for Mathematica, e.g. in package form
	->  the output data are readable by Cool`ReadData`	 

	There are Dif, Dat, CSV, Package and Math formats.
	Dif are assumed to be written by observations (rows) while
	Dat and .M are transposed, and give properties/variables (colums).
	A Math file will have the form {{name1, 2, 3.3, ..}, {name2 ...}....}
	while Dat and .M have the form "name1 = {2, 3.3, ...}" per record.
	Thus, reading Dat or .M allows immeadiately for operations like
	name1^name2 etcetera.
	The CSV format is line by line, elements separated by a separator.

	
	When there are more files per context, and they are big, then the 
	proposed routine is: DifBigToM. It calls:

	DifToMath:  		read Dif and produce an interim file
	MathToUnionDataList: read interim files into datalist 
	DataToDat:		put this in a datafile
	ToPackWithDat:  	use a package for putting a selection in a context,
					and the package calls the .dat

	For smaller files

	DifToMatrix		the Dif is read directly and put into a list
	ToPackAlone		the package contains all data (.moi)
	DifToPackage		directly
         DataToCSV		a list of lists put out as lines of elements
	DataToDif:		puts a list (of lists, but may be single) to a dif file


	NB. data read from package are contexted. E.g. "coni`datj"
	NB. Dif files differ. E.g. EXCEL (see the program)
	NB. procedures generally return True, unless some error
	NB. the procedures write packages that rely on DB (DataBase)
	selection as mentioned in the Cool`ReadData` package. 
	Calls are only written in text form. DB is not called.
	
	Subroutines:  
	- AdaptDif : to deal with blanks at end of strings within DIF files
	- UniqueName:  to deal with more occurences of variables 
	- Pralse[x_String] := (Print/Message[x]; Throw[False])  print, return False 
	- MemoryConstrainedQ[proc_, limit_] := True if memory constraint applied
	- CleanUp = ShortenUpToBlank for Dictionary[Variables] & into Dictionary 
	- ShortenUpToBlank = break text at first blank
         - WriteLine: Writing data in String format
	- Wrtnum, Wrtval, Wrtstr & Wrtbot for DataToDif
	Imported:
	- DataPackage = dir <> ps <> ".m"
	- Pal[a, items ...] write package lines
	Note:
	- Dif files have a preferable extension ".dif"
	- Dat files have a preferable extension ".dat"

It is suggested that the first variable name is "nr" and that the number code contains a context reference, like cc679 belongs to cc`nr. This first allows the automatic definition of the context. Secondly, if there are more (dif) files, then one can check more easily whether the properties concern the same numbers. The user can control such checks with the parameter DbaseTest, which is the only variable taken from the Cool`Dbase` package.

		Missing data are represented by Null.
		Output variables may need to be Cleared before usage;
		e.g. output lists must be "atoms" when entered as argument,
		and this is denominated in the "usage statement" by "_aioList"
		(Atom in, output List)
		Note that the ` reads badly in "dif`", so therefor this package is
		called "difdata`".
 *********************************************************************)





Needs["Cool`Common`"]

BeginPackage["Cool`DifData`",  Join[{"Cool`Dbase`", "Cool`Dictionary`", "Cool`PackLine`"}, $Cool]]

DataToCSV::usage ="DataToCSV[dir_String, filout_String, data_List, csv_String:";"]\nThe data list must be a list of lists. These lists are put on file, line by line. The elements are separated by the csv_String ('Comma Separated Variable)."

DataToDif::usage ="DataToDif[dir_String, filout_String, data_List]\nThe data are put on a dif-formatted file, of the EXCEL type. Null's are changed into empty strings. The data may be a vector or a list of (unequally sized) vectors."

DataToDat::usage = "DataToDat[dir_String, filout_String, data_List, varnames_] \n puts the data in a file.  Input data are {{name1, 2,2,},{name2,,},...}. You may suppy a list of variables, otherwise it is put out. The structure of the datafile is var = values. The first item in filout is VariableNames = {name1, name2,...}. Then follow OutputForm[varnames[[i]]] = Rest[data[[i]]]. This output file can be read with ReadList[filename] or ReadData[...]. The preferable filename extension of filout is .dat"

DifToMatrix::usage = "DifToMatrix[\"Dif_filename\"] \n if it is a Dif file then the whole file is processed and put into a List"

DifToMath::usage = "DifToMath[\"DIF_filename\", \"Mathematica_filename\"] \n line by line transformation of DIF file to a Mathematica-format file"

DifToPackage::usage = "DifToPackage[ begin_Integer,\n NFilesPerContext_Integer,\n diroriginal_String,\n dirinterim_String,\n dirpackages_String,\n memorylimit_Integer,\n CleanUp_logical,\n UpdateDictionaryQ_logical] \n the numbered DIF-files are first transformed line by line to Mathematica files, and then combined to a package"

DifBigToM::usage = "DifBigToM[ begin_Integer,\n	NFilesPerContext_Integer,\n diroriginal_String,\n dirinterim_String,\n dirpackages_String,\n memorylimit_Integer,\n CleanUp_logical,\n UpdateDictionaryQ_logical] \n the numbered DIF-files are first transformed line by line to Mathematica files, and then combined to both a datafile and a package"

MathToUnionDataList::usage = "MathToUnionDataList[ dir_String, filelist_List,\n varnames_aioList,\n tran_aioList, CleanUpQ_,\n UpdateDictionaryQ_  ] \n the filelist[[i]] in dir are combined into a dataset in memory. If CleanUp then only-Nulls are reduced and dictionary variables are abbreviated. If UpdateDictionaryQ then the dictionary is Updated"

ToPackWithDat::usage = "ToPackWithDat[dir_String, ps_String, varnames_List,  flndat_:Automatic]\n   makes package which reads data with ReadData[ ... flndat ...]. The package is ps<>.m and automatic flndat is ps<>.dat"

ToPackAlone::usage = "ToPackAlone[dir_String, data_List, varnames_, (r___Rule) ]\nmakes package with OutputForm[variableName[[i]]] = Rest[data[[i]]]. By default the context` is taken as the first two positions of data in \"nr\". Otherwise use the option DbaseTest. If varnames is an Atom, then it is put out too (as a list)."

PackageFirst::usage = "PackageFirst[a_, ps_, varnames_] writes the first part of the package using the package context name and the variable names."
Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
