(*******************************************

	Cool`Economic`Macro`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1998 Thomas Cool *)
(* :Package Version: 1.01,  July 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function.
 *)

Needs["Cool`Common`"]

BeginPackage["Cool`Economic`Macro`", Join[{"Cool`CES`", "Cool`Economic`Common`"}, $Cool]]

(*unemployment is a rate*)
(*rate of interest and unemployment as perunage and not percentage*)

DemandPerCapita::usage = "Keyword for Demand / Population"

Debt::usage = "Keyword for the national debt"

Deficit::usage = "Keyword for a negative budget surplus. Hence, if the budget is in surplus of -$10 bn, then the deficit is $10 bn; and if the budget is in surplus of $10 bn, then the deficit is -$10 bn"

ExampleModel::usage = "ExampleModel[{x___},y___] is a SolveFrom application that accepts rules or lists of rules x. These must set some parameters or variables in the equations of the model, such that a solution for the others can be found. These equations are taken from Options[ExampleModel], with the default setting Equations -> $Surprises. The additonal y can be a list of variables that are to be eliminated, or options for Solve or SolveFrom. E.g. try ExampleModel[{}]"

Exports::usage = "Keyword for the flow of goods and services out of the 
country of interest"

ExternalAccount::usage = "Keyword"

GDP::usage = "Keyword for Gross Domestic Product"
GNP::usage = "Keyword for Gross National Product"

Employment::usage = "Keyword, use as Min[Labour[Supply], Labour[Demand]]"
Inflation::usage = "Keyword for RateOfGrowth[PriceLevel], measured in perunages"

Imports::usage = "Keyword for the flow of goods and services into the country of interest"

IS::usage = "IS[Y] gives the rate of interest r as a function of Y, assuming only partial equilibrium on the commodity market. Has to be defined by the user. The start up value fits the start up model structure and parameter values"

LM::usage = "LM[Y] gives the rate of interest r as a function of Y, assuming only partial equilibrium on the money market. Has to be defined by the user. The start up value fits the start up model structure and parameter values"

ISLMModel::usage = "ISLMModel[{x___}, y___] is a SolveFrom application that accepts rules or lists of rules x. These must set some parameters or variables in the equations of the model, such that a solution for the others can be found. These equations are taken from Options[ISLMModel], with the default setting Equations -> $ISLMEquations. Additonal y can be a list of variables that are to be eliminated, or options for Solve or SolveFrom. E.g. try ISLMModel[{}]"

ISLMPlotPrepare::usage = "ISLMPlotPrepare[f:IS, g:LM] prepares a plot of ISLM. The plotting domain is found by solving IS[Y] == LM[Y], and LM[Y] = .2. One can use the result of this routine either as an inspiration for own plotting domains and ranges, or do Plot @@ ISLMPlotPrepare[]" 

LabourMarket::usage = "LabourMarket[{x___}, y___] is a SolveFrom application that accepts rules or lists of rules x. These must set some parameters or variables in the equations of the model, such that a solution for the others can be found. The equations are taken from Options[LabourMarket], with the default setting Equations -> $LabourMarketEquations. Additonal y can be a list of variables that are to be eliminated, or options for Solve or SolveFrom"

(*Not in Private, to directly 'declare' variables*)

ListOfSymbols[ExampleModel] = 
  {{RateOfInterest, r}, {PriceLevel, Py}, {Inflation, p},
  {Lag["PriceLevel"], "Py[-1]"}, 
  {Real["RateOfInterest"], R}, 
  {Expected["Real[RateOfInterest]"], Re}}// Sort // MatrixForm;

ListOfSymbols[ExternalAccount] = 
  {{Exports, X}, {Imports, IM}} // Sort // MatrixForm;

ListOfSymbols[ISLMModel] = 

  {{Consumption, C}, 
  {Bonds[Demand, Real], RDB}, 
  {Bonds[Supply, Nominal], NSB}, {Lag["Debt"], "Debt[-1]"},
  {DisposableIncome, YD}, {Government, G}, 
  {InterestFloor, rmin}, {Investment, J}, {Liquidity, L}, {Money, M}, 
  {NetExports, NX}, {RateOfInterest, r}, {Saving, S}, 
  {Taxes, T}, {TransferIncome, TR}, 
  {Velocity, V}, 
  {Wealth, NPW}, {Income, Y}, {PriceLevel, Py}, {Inflation, p},
  {Lag["PriceLevel"], "Py[-1]"}, 
  {Autonomous["Consumption"], Caut}, 
  {Autonomous["Investment"], Jaut}, 
  {Parameter["ConsumptionFunction"], c}, 
  {Parameter["Taxes", "Income"], tau}, 
  {Parameter["LiquidityPreference", "Interest"], h}, 
  {Parameter["LiquidityPreference", "Transaction"], k}, 
  {Parameter[ "Investment", "RateOfInterest"], b}} // Sort // MatrixForm;

ListOfSymbols[LabourMarket] = 

  {{Employment, L}, 
  {Expected["Inflation"], pe}, 
  {LabourIncomeQuote, LIQ}, {LabourProductivity, yl}, 
  {Price, Pp}, {Income, Y}, {PriceLevel, Py}, 
  {WageInflation, w}, {Wage, W}, 
  {Unemployment, u}, {UnemploymentLevel, U}, {Labour[Supply], LS}, 
  {Vacancy, v}, {Labour[Demand], LD}, 
  {Parameter["WageInflation", "Taxes"], rho}, 
  {Lag["Wage"], "W[-1]"}, 
  {Lag["WageInflation"], "w[-1]"} } // Sort // MatrixForm;

ListOfSymbols[ValueAdded] = 

  {{Employment, L}, {Inflation, p},
  {Intermediate, Xm}, 
  {MarkUp, m}, {Income, Y}, {PriceLevel, Py}, 
  {MaterialsPrice, Pm}, {Price, Pp}, {Production, Yp}, 
  {Wage, W}, 
  {Capital["Income"], rPkK}, 
  {Lag["PriceLevel"], "Py[-1]"}, 
  {Parameter["Intermediate", "Production"], theta}, 
  {ValueAdded, VA}} // Sort // MatrixForm;


LIQRule::usage = "LIQRule[Line || Log, {alpha, beta, gamma}] gives a rule for the LabourIncomeQuote as already solved from the LabourMarket[] using the relevant PhillipsCurve and setting rho = 0. See these references. Be aware that this rule only holds for these equations ! Note that nonzero values of rho can be used by setting alpha = alpha' + rho tau"

LiquidityPreference::usage = "LiquidityPreference[Y, r, {k:.2, h:.02, rmin:.01}] gives (1 + h /(r - rmin)) k Y as the real demand for money, for income Y and rate of interest r. Parameters are transaction parameter k (default .2), the minimal rate of interest rmin (default .01), and a sensitivity parameter h (default .02). This form of liquidity preference is more agile than linear L = k Y - h r. Note how this behavioural relationship relates to the accounting relations L = k' Y,  M V = Py Y,  M / Py = (1/V) Y"

MacroSymbols::usage = "The list of compact symbols that are known by ToMacroSymbols at start up time. These symbols are protected by default, since one is inclined to set c = 0.9 etcetera while this is not necessarily the intended use of these symbols. To unprotect these symbols, evaluate Unprotect @@ MacroSymbols. To protect them again, evaluate Protect @@ MacroSymbols"


NAIRU::usage = "NAIRU[expr, u] simply solves u from expr == 0, regarding expr as a function of u. Let us discuss when this solution is the Non-Accelerating-Inflation Rate-of-Unemployment.\n(1) If p is inflation and p[-1] was last year's inflation, then the acceleration of inflation is p - p[-1]. (Note that - properly - this should be divided by p[-1] again, but this is not common in the literature.)\n(2) Wage inflation will be set as w = pe + PhillipsCurve[LIQ, u], for pe anticipated inflation, u unemployment and LIQ the labour income quote in total value added (as an indication of profits). \n(3) Let f[] give the functional relation of wage inflation to general inflation, then: p = f[w]\n(4) Hence, solve 0 == p - p[-1] == f[pe + PhillipsCurve[LIQ, NAIRU]] - p[-1] for NAIRU\n(5) Since the latter is difficult, due to f[], one often is satisfied with the NAWIRU (the W for wage) as solved from:\n     w - w[-1] == pe + PhillipsCurve[LIQ, NAWIRU] - w[-1] == 0"

NAWIRU::usage = "Keyword, see NAIRU"

NetExports::usage = "Keyword, equals Exports - Imports"

Participation::usage = "Keyword, Labour / Population"

PhillipsCurve::usage = "PhillipsCurve[Line || Inverse || Log, LIQ, u, {alpha, beta, gamma}] gives a contribution to the development of wage inflation due to the tensions on the labour market, with u unemployment and LIQ the labour income quote in total value added (as an indication of pofits). The other are parameters, defaults for Log {-.35, -.1, -.1}, and defaults for Line {.10, -.03, -2}. The Inverse version has no default. Note: In both cases, LIQ and u can be seen as measured against some optimal value, e.g. Log[u/u*], but these additional parameters are here solved out into the constant. Note: Some would restrict the Phillipscurve effect in the wage equation to u only, but it is better to include LIQ"

PriceLevel::usage = "Keyword"

PropensityToConsume::usage = "Keyword. Understood as the marginal propensity to consume, it is Keynes's term for the proportion of additional income that is consumed"

Unemployment::usage = "Keyword for 1 - Employment / Labour[Supply] (measured in perunages)"

SelectSolutionRules::usage = "List of rules to select useful solutions. Default eliminated are solutions that contain negative RateOfInterest and PriceLevel"

ToMacroSymbols::usage = "A list of rules to change macro keywords into short variables. Use eluR[ToMacroSymbols] for the reverse"


UnemploymentLevel::usage = "Keyword, Labour[Supply] - Employment"

Vacancy::usage = "Keyword for 1 - Employment / Labour[Demand] (measured in perunages)"

ValueAdded::usage = "ValueAdded[{x___}, y___] is a SolveFrom application that accepts rules or lists of rules x. These must set some parameters or variables in the equations of the model, such that a solution for the others can be found. The equations are taken from Options[ValueAdded], with the default setting Equations -> $ValueAdded. Additonal y can be a list of variables that are to be eliminated, or options for Solve or SolveFrom.\nNote that this small model explains that Inflation is regarded here as the relative price change of the PriceLevel, and the latter is the price of Income (Real Value Added). Hence, gross output (Production) has a different deflator, Price"

$ISEquations::usage = "List of equations used to construct the commodity market partial equilibrium I = S curve"

$LMEquations::usage = "List of equations used to construct the money market partial equilibrium L = M curve"

$ISLMEquations::usage = "List of equations for a textbook I = S & L = M model, and the default option value for ISLMModel[]"

$ISLMParameters::usage = "An example list of parameter settings for $ISLMEquations"

$LabourMarketEquations::usage = "List of equations for a textbook labour market model, and the default option value for LabourMarket[]"

$LabourMarketParameters::usage = "An example list of parameter settings for $LabourMarketEquations"

$Surprises::usage = "A list of some surprise equations, see Surprise"

$ValueAddedEquations::usage = "List of equations"
$ValueAddedParameters::usage = "An example list of parameter settings"

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
