(*********************************************************************

	Cool`Estimate`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1996 Thomas Cool *)
(* :Package Version: 2.0,  June 1997.*)
(* :Mathematica Version: 3.0 *)
(* :Keywords: Statistics, Econometrics, Estimation   *)
(* :Limitation: *)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)

Needs["Cool`Common`"]

BeginPackage["Cool`Estimate`",  Join[{"Statistics`NonlinearFit`",
"Statistics`LinearRegression`", "Cool`Statistics`Common`", "Cool`Matrices`",
"Statistics`DescriptiveStatistics`", 
"Statistics`Common`MultivariateCommon`",
"Statistics`Common`RegressionCommon`"}, $Cool]]

Estats::usage = "Estats[equation_Equal, vardata_List, coefs_List, estimate_List, opts___Rule] computes basic regression statistics for the equation given the estimate"

Estimate::usage = "1) Estimate[y_Symbol == rhs, {y -> data1, var2 -> data2, ...}, coefficients_List, opts___Rule] estimates coefficients of a single-equation model using the data.\n2) Estimate[{eq1, eq2,...}, {var1 -> data1, var2 -> data2, ....}, coefficients_List, opts___Rule] estimates a system of equations by constructing the proper single-equation form and dataset.\nThe routine calls NonlinearRegress and provides additional statistics using Estats. The routine uses the rhs's, does not solve for a reduced form, and thus variables may contain errors. The coefficients list and options are passed on to NonlinearRegress. The coefficients e.g. contain symbols and starting values {{coef1, start1}, {coef2, start2}, ...}. All variables must have the same number of observations. Option CovarianceMatrix -> True (default) gives an estimated covariance matrix using the inverse Hessian.\n3) When input option Estimate -> False, then no estimation is tried, and only T-values are computed for the given start-coefficients. When option Out -> False then the original output of NonlinearRegress is given."

ErrorsInVariable::usage = "ErrorsInVariable[0 == rhs, errorvariable, vardata_List, coefs_List, opts___Rule] estimates coefficients of a single-equation model using the data, when the rhs cannot be solved explicitly in terms of the errorvariable. It uses FindMinimum on the sum of squared errors, with\n   error = (errorvariable - ev) + lambda rhs[ev].\n   In the rhs the errorvariable is replaced with its true value ev. Internally, ev is taken as an unknown parameter; thus each observation gives an ev[i]. Lambda must be given as the first entry in coefs. If option Print -> True then error is printed. Other inputs are as in Estimate."

InverseHessian::usage = "InverseHessian[f_Equal, vardata_List, coefs_List, coefestimationrule_List] gives the inverse hessian of the sum of squared error at the estimated point, determined by formal differentiation."

NumberOfEquations::usage = "Keyword for the number of equations in a model. See Estimate."

ModelY::usage = "Keyword for the collection of all endogenous variables in Estimate"

ReducedFormQ::usage = "ReducedFormQ[x] where x can be an equation or a list of equations. True iff all left hand sides contain single variables (called endogenes) and iff all endogenes only occur in the lhs's."

CovarianceMatrix::usage = Key[AddedUsage, CovarianceMatrix, "1) Input option, True or False, controls the generation of output.\n2) Output option gives the estimated matrix for the estimated coefficients."]

StandardDeviation::usage = Key[AddedUsage, StandardDeviation, "Output option, gives the estimated standard deviation of the estimated coefficients (so that coef/stdev is the Student t-value)."]

RSquared::usage = Key[AddedUsage, RSquared, "Output option, gives the uncentered RSquare, unadjusted for the degrees of freedom."]

AdjustedRSquared::usage = Key[AddedUsage, AdjustedRSquared, "Output option, gives the squared centered correlation coefficient adjusted for the degrees of freedom."]

Options$Estimate = {CovarianceMatrix -> True, Estimate -> True, Out -> True}

Options[Estimate] = Options$Estimate

NonlinearFitOld::usage = "NonlinearFitOld[...] gives the Mathematica 2.2.2 version of NonlinearFit. It calls NonlinearRegress, and only puts out the estimated coefficients"

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
