(******************************************************************

	Cool`Finance`  package


 *******************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1994/95 Thomas Cool *)
(* :Package Version: 3.0, June 1997*)
(* :Mathematica Version: 3.0 *)
(* :Keywords:     Finance, Economics, Date, Calendar *)
(* :Limitation:*)
(* :Discussion: I don't consider myself to be a Finance Expert. This package
is very much based on the WRI Finance Pack. It uses the FP packages, and the new routines are based on the examples in the FP notebooks.*)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)


(* Time and FlatRate for up/downward compatibility *)
Needs["Cool`Common`"]

BeginPackage["Cool`Finance`", Join[{"Cool`Time`", "Cool`FlatRate`", "Finance`Bonds`", "Finance`Common`", "Finance`Calendar`", "Finance`InterestRates`", "Finance`CashFlows`", "Finance`Examples`", 
"Miscellaneous`Calendar`"}, $Cool]]

F$ContextPath::usage = "F$ContextPath[] adjusts the $ContextPath such that the order is { ... Cool`Finance`,..., Cool`Time`, ..., Finance`Calendar`, ..., Miscellaneous`Calendar`,...}."

Options[AnnualToDayRate] = {Count -> 360, Factor -> 10000.}
Options[DayToAnnualRate] = "Use the options of AnnualToDayRate, which control both functions"

AnnualToDayRate::usage = "AnnualToDayRate[aR, opts] changes an annual interest rate aR into a day rate. Default option Count -> 360 takes 360 days per annum; alternatives are 365, 366, or a year, like 1990 (and then taking account of leap years). Default option Factor -> 10000. gives the result as base points (percents of percents) per day. If the switch of rates per one (years) to rates per 10000 (days) is confusing, set Factor -> 1."

AssetBeta::usage = "Keyword, gives the Beta of a CAPM regression"

BondToCashFlows::usage = "BondToCashFlows[bond_Bond, FDate_, dated_:True] changes the bond into a CashFlows object, if standard then dated, and if dated=False then in cumulated period form"

CashStocks::usage = "CashStocks[{{time1, amount1}, {time2, amount2}, ... }] is an object for a cash register, the cumulation of cash flow"
CashCumulate::usage = "CashCumulate[x] cumulates a CashFlows object into a CashStocks object"
CashDifference::usage = "CashDifference[x] changes a CashStocks object into a CashFlows object"

DayToAnnualRate::usage = "DayToAnnualRate[dR, opts] changes a day interest rate into an annual rate. By default, day rates are in base points while annual rates are per one. For options, see AnnualToDayRate."

DatedDiscountFactors::usage = 
"DiscountFactors[{FDate1, factor1}, {FDate2, factor2}, ... }] is an interest rates object represented by financial dates and discount factors. The initital date must have discount factor 1."

DayToDayInterestFactor::usage = "DayToDayInterestFactor[begindate, enddate, interestRate, opts___Rule] gives the compounded interest factor from one day to another. The input interestRate is assumed to be annual, and the dayly rate is computed by AnnualToDayRate[interestRate, opts] and its defaults."

FinanceClasses::usage = Key[AddedUsage, FinanceClasses, 
	"CashStocks\nDatedDiscountFactors\nPortfolio"]

FinanceGO::usage = "FinanceGO[FinanceObject] makes a Graphics Object of a Finance Object"

InterpolateDF::usage = "InterpolateDF[d_DiscountFactors] interpolates the whole range of the discount factors object"

IntervalSumFR::usage = "IntervalSumFR[object_ForwardRates] cumulates the subperiods of the forward rates to find the total duration"

SingleInterpolatedDF::usage = "SingleInterpolatedDF[{{t, f}, {T, F}}, time_] \n gives a single interpolation for discount factors that may be in symbols" 

FOToPeriods::usage = "FOToPeriods[d_, ini_:Automatic] transforms a dated finance object into an object using actual days in between, following standard Mathematica dating. The first date in the FO wil be used as the initial date, unless another date is specified."

Options[PortfolioAnalysis] = {Plot -> True, Print -> True, PlotStyle -> PointSize[.02]};

Portfolio::usage = "Finance Object. Portfolio[fo1, fo2, ...] collects various finance objects."

PortfolioAnalysis::usage = "PortfolioAnalysis[data, opts] determines the means and covariances of the data, and the Markowitz Efficient Frontier. Input data = {r1, r2, ...} where ri is the list of rates of returns of asset i. The efficient frontier is generated in parametric form, as EfficientFrontier[x]. Options[PortfolioAnalysis] control printing and plotting."

PortfolioWeights::usage = "PortfolioWeights[mean_List, covariance_?MatrixQ, W_] supplies the weights for PortfolioMean and PortfolioVariance. W will normally be a Symbol."

RiskFreeRate::usage = "Keyword, gives the risk-free rate of a CAPM regression"

SecurityMarketLine::usage = "SecurityMarketLine[x] gives the line (1-B) RFR + B x of a CAPM regression. This line is generated by SecurityMarketRegression[...]."

SecurityMarketRegression::usage = "SecurityMarketRegression[y_List,z_List,opts___Rule] regresses z = coefficient y + constant. That line becomes available as SecurityMarketLine[x]. Options[PortfolioAnalysis] control printing and plotting. Output is {AssetBeta -> coefficient, RiskFreeRate -> constant / (1-coefficient)}.\nNote: when z is a list-of-lists, then the regression takes the asset beta's against the asset mean returns. The SubValues -> ... option in output gives the results of the sub-regressions."

ShowFO::usage = "ShowFO[x] shows Finance Object(s), single or list of those"

UsefulCashFlowsDates::usage = "UsefulCashFlowsDates[bond_Bond, settlement] gives the cashflow dates including maturity and excluding settlement if that falls on a cashflow date"

Value::usage = Key[AddedUsage, Value, "Value[Portfolio[x__], day, rates] gives the value of the portfolio at the settlement day for the given rate. Bonds are first changed into cash flows to get the proper value."]
Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
