(*******************************************

	Cool`Freight`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1998 Thomas Cool *)
(* :Package Version: 1.08,  July 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function.
 *)

Needs["Cool`Common`"]

BeginPackage["Cool`Freight`", Join[{"Miscellaneous`Units`", "Cool`Physics`", "Cool`Databank`", "Cool`Transport`", "Cool`Statistics`Common`", "Cool`Economic`Common`"}, $Cool]]


LoadFactor::usage = "Keyword to indicate the load factor = TonneKilometer[] / TonneKilometers[Distance]\nThis assumes actual distances and not operating or capacity distances.\nFor a single Lift the distance is the same, and then it is TonnesLifted[] / Weight[Capacity]"

Operating::usage = "Keyword to indicate operating conditions"

Empty::usage = "Keyword for properties of empty runs"

Docking::usage = "Keyword for the (un-) docking time taken by vehicles"

Forty::usage = "Keyword for forty feet containers";

Freight::usage =  "Keyword for databank usage, combines Trip and Vehicle\nFreight[i] creates a Freight object i\nFreight[t, v] combines Trip object t with Vehicle object v"

FreightExplain::usage = "FreightExplain[x] explains terms for Trip and Freight (Lift is slightly different) for x = All, Duration, Distance, TonneKilometer, Weight, Factor"

FreightModel::usage = "FreightModel[{x__}, y___List, opts___Rule] is a SolveFrom application, using FreightEquations"

FreightEquations::usage = "List of equations for FreightModel"

FreightSymbols::usage = "List of short symbols in ToFreightSymbols, protected"

Haul::usage = "Keyword for databank usage\nHaul[i] creates an object Haul[i], consisting of the loading weight and time at the start, the Lift[i], and unloading weight and time at the end. It is assumed that the vehicle may already be partly loaded, so that the Lift[i] and unloading may concern more weight than loading. Similarly, unloading may be less than loading"

Handling::usage = "Keyword for properties of handling cargo"

Idle::usage = "Keyword for properties of idle vehicles"

Lift::usage =  "Keyword for databank usage\nLift[i] creates a Lift object i \nNote the states of a vehicle: Run, Empty, Idle, Handling.
\nHandling is the time for a Lift with 0 distance and 0 weight.
\nEmpty is the time with a Lift with 0 weight.
\nIdle is determined from the vehicle data"

LiftToHaul::usage = "LiftToHaul[x] turns a lift object into a haul object. A lift only concerns the transport, the haul includes loading and unloading"

Trip::usage =  "Keyword for databank usage, depends on Lift\nTrip[i] creates a trip object i"

LiftsToTrip::usage = "LiftsToTrip[x_List] transforms a list x of Lift objects into a Trip object. Assumed is that there is only a loading moment at the beginning of the first lift, and after that there is only unloading. The tonnes in transport are modelled as the maximum in all lifts"

HaulsToTrip::usage = "HaulsToTrip[x_List] transforms a list x of Haul objects into a Trip object. Assumed is that the first haul is the first of the trip, so that an already existing load is added to the tonnes in transport. The first element of Results[HaulsToTrip] = {existing load, loaded - unloaded, remaining load = existing + loaded - unloaded}"


ToHandling::usage = "ToHandling[hs_List] determines a handling lift object from list haul objects"

HaulsToLifts::usage = "HaulsToLifts[hs_List] makes a list of lift objects from a list of haul objects hs, while adding the implied handling lift object"

ThroughputModel::usage = "ThroughputModel[{x__}, y___List, opts___Rule] is a SolveFrom application, using the equations:
\n     Throughput == Twenty + 2 Forty                               measured in TEUs
\n     TEUfactor == (Twenty + 2 Forty) / (Twenty + Forty)
\n     TEU == Throughput / Vehicle	                                Vehicle the average number of vehicles, giving average load in TEU's
\n     LoadFactor == TEU / Volume
\n     Load == (Twenty + Forty) / Vehicle                           average load in containers
\n     Duration[\"Load\"] == Load * CycleTime / N                   N the number of cranes, CycleTime in hours
\n     Speed == Load / Duration[\"Load\"]
\n     Duration[\"Vehicle\"] == Duration[\"Load\"] + Docking        average handling duration, in hours
\n     Utilisation[\"Vehicle\"] == 
		Duration[\"Load\"] / Duration[\"Vehicle\"]
\n     Duration[\"Year\"] == Vehicle Duration[\"Vehicle\"]          total duration taken by vehicles
\n     Utilisation[\"Year\"] == Vehicle Duration[\"Vehicle\"] / Operating
\n     Operating == Open Year                                   Open would give the hours per day
"



Throughput::usage = "The number of TEU's handled in a year";

TEUfactor::usage = "The TEU per container:\nTEUFactor == (NTwenty + 2 NForty) / (NTwenty + NForty)";

TEU::usage = "Twenty feet container Equivalent Unit, a unit of handling";

Twenty::usage = "Keyword for twenty feet containers";

ToFreightSymbols::usage = "List of rules to change FreightEquations etc. into shorter symbols"

TonnesLifted::usage = "Keyword for the observed tonnes lifted"

TonnesInTransport::usage = "Keyword for the actual (unique) tonnes in transport = Handling[Factor] * TonnesLifted"

Utilisation::usage = Key[AddedUsage, Utilisation, "Keyword to indicate TonneKilometer[] / TonneKilometer[Capacity]\nDiffers from LoadFactor since the capacity tonnekilometers are taken and not the operating tonnekilometers"]

Vehicle::usage = "Keyword for databank usage\nVehicle[i] creates a vehicle object i"


ListOfSymbols[FreightModel] = 
{
{Duration[Weight],       DL},
{Duration[Empty],        DE},
{Duration[Handling],     DH },
{Duration[Idle] ,        DIdle},
{Duration[],             DBusy},
{Duration[Operating],    DO},
{Duration[Capacity] ,    DC },

{Distance[] ,           VK },
{Distance[Weight],     LVK},
{Distance[Average],      AL},
{Distance[Capacity] ,    KCap},
{Distance[Empty] ,       KE},
{Distance[Operating]  ,  KO},
{Distance[Integrate],    K},

{TonneKilometer[]   ,     TK },
{Operating[Capacity] ,     OCTK },
{TonneKilometer[Capacity], TKCap },
{TonneKilometer["Actual", Capacity], ACTK },

{Weight[Capacity] ,       CC},
{TonnesLifted[],                TL},
{Weight[Average],         TAv },
{TonnesInTransport[],       T},

{Handling[Factor] ,    h},
{Handling[Average] ,      hAv},
{LoadFactor,             LF},  
{Utilisation[Weight],     qT},
{Utilisation[Distance],   qK},
{Utilisation[Capacity],   qC},

{Speed[Capacity],         sCap},
{Speed[Average],          sAv}} // Sort // MatrixForm

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
