(*********************************************************************

	Cool`Inference`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1995 Thomas Cool *)
(* :Package Version: 2.0, August 1995.*)
(* :Mathematica Version: 3.0 *)
(* :Keywords: logic   *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)

Needs["Cool`Common`"]

BeginPackage["Cool`Inference`",  Join[{"Cool`Logic`"}, $Cool]]



(*** axioms and metarules ***)

ModusPonens::usage = "ModusPonens, axiom\n ((Implies[p, q]) && p) :> q"

IfToIf::usage =  "IfToIf axiom\n  ((p \[Implies] (q \[Implies] m)) :> ((Implies[p, q]) \[Implies] (p \[Implies] m)))"

IfToAnd::usage = "IfToAnd axiom\n   (((Implies[p, q]) \[Implies] m) :> ((!p \[Implies] m) && (q \[Implies] m)))"

EFSQ::usage = "Ex Falso Sequitur Quodlibet:\nFrom False follows anything you want, axiom\n (!p \[Implies] (Implies[p, q])) :> True"

IfTransitive::usage = "IfTransitive axiom\n (Implies[p, q]) && (q \[Implies] r) :> (p \[Implies] r)"

SelfImplication::usage = "SelfImplication, axiom \n (!p \[Implies] p) :> p   and   (p \[Implies] p) :> True"

AndSymmetric::usage = "AndSymmetric = (x_ && y_) :> (y && x), an example metarule"

OrSymmetric::usage = "OrSymmetric = (x_ || y_) :> (y || x), an example metarule"

IfAntiSymmetric::usage = "IfAntiSymmetric = ( x_ \[Implies]  y_) :> (!y \[Implies] !x ), an example metarule"

(*****  inference machine *****)

Infer::usage ="Infer is a list of replacement rules for solving logical statements. It is standardly based on the axioms SelfImplication, ModusPonens, EFSQ, IfTransitive, and IfToAnd, while using metarules AndSymmetric, OrSymmetric and IfAntiSymmetric. It presumes AndOrEnhancement. Use the rule InferAndOr, if you want AndOrRules included.\nFor a logical statement x, where \[Implies] plays the role of implication, give x //. Infer"

InferQ::usage = "InferQ gives suggestions on the use of the package\nInfer is a list of replacement rules for solving logical statements"

InferenceMachine::usage = "Infer = InferenceMachine[...] has already been set. The pseudo-options are:\nHoldAll -> {axioms taken as they are},\nHold ->  {those axioms that already conclude to True or False; expandable},\nAxiomToTrue -> {those axioms that don't do this and you want to add 'axiom :> True'; expandable},\nExpandAxiom -> {other axioms which will be expanded},\nPlacedProperties -> {And -> {...}, If -> {...}, ...} metarules that should hold for any instance of the operators."

InferAndOr::usage  = "AndOrRules ~Join~ Infer"

(******************)

AndEmbedding::usage = "1) AndEmbedding[r] embeds p_ && q_ within BlankNullSequences, while the pattern-names are added to the last part (assumed to be the conclusion).\n2) Option to do so in ExpandAxiom." 

AxiomToTrue::usage = "AxiomToTrue[x, y_:Implies, adjustable_:True] = (x /. RuleDelayed -> y) :> True. Constructs the rule: whole axiom :> True. (If adjustable, then the conclusion section is patterned if the premiss was.)"

Deny::usage = "Deny[x] gives a rule for denial of x: x -> Not[x]; x can be a list"

DenyPattern::usage = "DenyPattern[pattern, x, prn_:True] replaces x_ with !(x_) within pattern, and x with !x (for conclusions). Default with input pattern printed too."

Options[ExpandAxiom] = 
	{MachineForm -> True, First -> True, LogicalPattern -> True,
	Rule -> False, AndEmbedding -> True, Union -> True, 
	Implies -> "", And -> "", Or -> "", Xor -> ""};

ExpandAxiom::usage = "ExpandAxiom[axiom, opts___Rule] applies the metarules for Implies, And, Or & Xor, for all separate occurrences of these operators.\nThe metarules must be given as e.g. Implies -> name, or Implies -> {name1, name2, ...} where namei are Strings. ($namei will be the copy for the occurrences.)\nDefault options: MachineForm -> True applies MachineForm to the axiom first, First -> True means that the metarules only apply to the premiss, LogicalPattern -> True gives patterns in output, Rule -> True replaces Implies with Rule,  AndEmbedding -> True embeds p && q within BlankNullSequences, Union -> True flattens the result and thereby removes possible duplications. Also given are required default rules for Implies, And, Or & Xor."

MachineForm::usage = "MachineForm[x] = UnPattern[x] /. Rule -> Implies\nAxioms are internally set to MachineForm, to facilitate matches. See ToRule, MetaRuleForm and ObjectForm."

ObjectForm::usage = "ObjectForm[x] = UnPattern[x] /. RuleDelayed -> Rule. Transforms into object language form, including :>. See MachineForm."

MetaRuleForm::usage = "MetaRuleForm[x] = LogicalPattern[MachineForm[x]]\nNot automatically applied within ExpandAxiom. See MachineForm."

ToRule::usage = "ToRule[x] = LogicalPattern[x] /. Implies -> Rule. Transforms object language expressions into rules. See MachineForm."

PlacedMetaRule::usage = "PlacedMetaRule[name_String] subroutine for ExpandAxiom and PlaceProperties, derives a placed metarule from the general metarule"

PlacedProperties::usage = "PlacedProperties[len, x] subroutine for ExpandAxiom, gives each placed operator its own list of metarules; len is the number of occurrences of the operator, and x gives the list of general metarules for that operator"

Place::usage = "Place[Operator, i] marks the place of the ith operator"

PlaceOperator::usage = "PlaceOperator[expr] gives each operator it place mark"

PlaceUndo::usage = "PlaceUndo = {Place[x_, y_] -> x} removes Place marks"

Options[LogicalPattern] = {First -> True}

LogicalPattern::usage = "1) LogicalPattern[x] changes a logical expression x into a logical pattern. Default option First -> True applies pattern only to the premiss.\n2) LogicalPattern -> True|False an option of ExpandAxiom for output format."


Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
