(*******************************************

	Cool`LP`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1997 Thomas Cool *)
(* :Package Version: 1.04,  August 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function.
 *)
(* :Literature: Hillier and Lieberman, "Introduction to operations research, Holden Day 1967; 
M. Carter, "Linear programming with Mathematica", in Varian (ed), "Computational economics and finance", Telos 1996, creates a set of routines that allow one to work in a more symbolic manner.*)

Needs["Cool`Common`"]

BeginPackage["Cool`LP`", Join[{}, $Cool]]

NLP::usage = "NLP[c, A, b] finds the vector x which maximizes the quantity c.x subject to the constraints A.x <= b and x >= 0. (NLP = Numeric Linear Programming, NP = Nonlinear Programming)"

NLPAnalysis::usage = "NLPAnalysis[c_List, A_List, b_List] calls NLP, SimplexTableau, RangeOfFeasibility and RangeOfOptimality"

RangeOfFeasibility::usage = "RangeOfFeasibility[] gives the ranges of feasibility of the constraints, i.e. the ranges over which the resources can vary while their shadow price remains the same. The information of the last call of NLP[...] is used.\n\nP.M. for the optimum:\nRangeOfFeasibility[i, v, invB] gives the relative range of feasibility of the ith constraint using vector v with the values of the basic variables and the matrix invB.\nRangeOfFeasibility[v, invB] gives all relative ranges of feasibility"

RangeOfOptimality::usage = "RangeOfOptimality[] gives the ranges of optimality for the last call of NLP - i.e. the ranges for which each coefficient in the objective function can change without affecting the solution.\nRangeOfOptimality[c_List, A_List, basis_List] does this for the given basis\nRangeOfOptimality[\"2D\", a, {c1, c2}, b] solves the relationship a <= -c1'/c2' <= b for alternatively c2' = c2 (giving the range for c1') and c1' = c1 (giving the range for c2'). In linear programming, -c1/c2 is a slope of the objective function. The coefficients must stay in the deduced ranges if the current solution is to remain optimal. Note: this deduction assumes c1, c2 >= 0"

SimplexTableau::usage = "SimplexTableau[c_List, A_List, b_List, basis_List] prints the Simplex Tableau for max c.x subject to A.x <= b and x >= 0. The list of basic variables must be entered as the column numbers in (A I), with as much numbers as there are rows in A. The Tableau contains these numbers in the first column. The first row of the Tableau prints 1 for basic variables and 0 for the non-basic variables. The top right hand value is the total value score. The second row contains the net values of the objective function, with negative values indicating inoptimality. The last column gives the values of the basic variables, with negative values indicating inoptimality. See Results[SimplexTableau]"



TransportLP::usage = "TransportLP[c_List, a_List, b_List, u_Symbol, v_Symbol] solves the transport problem. Allocations x = (xij) must satisfy column sums 1'x = a (demand) and row sums x.1 = b (capacity), and the optimand is minimal cost 1'(c * x).1. In case of overcapacity, add a dummy column with zero costs; and with undercapacity, a dummy row. Note that you must supply two undefined symbolic variables for the shadow price analysis"

AnalyseTransportLP::usage = "AnalyseTransportLP[c_List, a_List, b_List, u_Symbol, v_Symbol, x_List] is a subroutine of TransportLP that analyses the result of a ConstrainedMin solution. The input a, b, c and x are explained in the usage statement of TransportLP. The output gives the shadow price analysis. The uij and vij are taken such that cij = uij + vij for the allocated or chosen xij. Applying these values to the non-chosen xij, should give nonnegative cij - (uij + vij). Negative cells indicate a non-optimal solution. Note that you must supply two undefined symbolic variables for the shadow price analysis. These are used to create 'SymbolArray's"


TransportLPMatrixQ::usage = "TransportLPMatrixQ[x_List, a_List, b_List, test_:False] tests the matrix x, column sums a and row sums b for the transport LP problem. If x is the allocation matrix, set the test at Plus  or Print. Results[TransportLPMatrixQ] then contains the differences between the true column and row sums and a and b. If test is Print, then this is printed. If x is the cost matrix, set test at any other value (default). Output is True if the test passes, False otherwise"

ShadowPrices::usage = "ShadowPrices[c, A, basis] gives the implied shadowprices associated with the selected basis, see NLP"

Slack::usage = "Keyword"

Basis::usage = "Basis[A, basis] selects the columns in A for the given basis, see NLP"

ProjectLP::usage = "ProjectLP[c_List?VectorQ, A_List?MatrixQ, b_List?VectorQ, {i_Integer, j_Integer}] determines parameters c', A' and b' for problem NLP[c', A', b'] for variables i and j, assuming that the other variables are at the optimum of NLP[c, A, b]"

LPPlot::usage = "LPPlot[c_List, A_List, b_List, {i_Integer:1, j_Integer:2}, opts___Rule] determines c', A' and b' using ProjectLP[c, A, b, {i,j}], and evaluates LP2DPlot[-c', -A', -b', opts]"

ToCartersForm::usage = "ToCartersForm[c_List, m_List, b_List, z_Symbol, x_Symbol, s_Symbol, {xlabels___Symbol}, {slabels___Symbol}] transforms the NLP[c, m, b] problem into the format used by M. Carter 1996, using the symbol z for the objective function, x for the proper variables, s for the slack variables, xlabels for indexing x and slabels for indexing s. If the latter are empty, then numbers are taken"

Vogel::usage = "Vogel[c_list, a_list, b_List] applies Vogel's approximation method to the transport problem. Here c is the (n, m) matrix of cost coefficients, a is the n-vector of demands and b is the m vector of capacities. Results[Vogel] show how the allocations have been succesively. Note that the method only gives an initial solution. It is included here only for didactic reasons.\nVogel[x_List] gives the Vogel penalty value of that list, i.e. the difference from the lowest value to the next value. In all, -1 stands for an eliminated row or column"
 
Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
