(*********************************************************************

	Cool`List`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright since 1993 Thomas Cool *)
(* :Package Version: 2.11 Thomas Cool, July 1998.*)
(* :Mathematica Version: 3.00 *)
(* :Keywords:   handling of lists *)
(* :Limitation:*)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function. *)
(* :Comment: missing data represented by Indeterminate instead of Null*)


BeginPackage["Cool`List`", {"Cool`Context`","Cool`Common`"}]

AggregatorQ::usage = "AggregatorQ[x] gives True when x is an aggregator, i.e. a matrix of only 1's and 0's, and the column sums all equal 1"

Aggregate::usage = "Aggregate[x_, y_, levelspec_:{1}] aggregates y at levelspec. That is: transpose level to first, then x . y', then transpose back. TransposeTensor is used."

AllQ::usage = "AllQ[list, crit, levelspec_:{1}, opts___Rule] gives True if list is a nonempty List and all its elements cause crit to be True. The routine uses MapLevel. Levelspec (default 1) and opts are for Level. Alternative to Select, Cases & Count.  Note: a Hintikka observation is that the universal quantifier always requires a range, and therefor AllQ depends on a list. See ExistQ."

Angle::usage = "Angle[a_List, b_List] gives the angle between vectors a and b\nAngle[a_List] gives the angle of vector a with the horizontal plane"

Applist::usage = "Applist[x_, y_] := If[ListQ[y], x @@ y, x[y]] does not replace a useful Head that is not a list"

ArgMaxQ::usage = "ArgMaxQ[x_, i_Integer] tests whether the ith position in x equals Max[x]"

ArgMinQ::usage = "ArgMinQ[x_, i_Integer] tests whether the ith position in x equals Min[x]"

Average::usage = "Average[lis] gives the average value\nAverage[lis, n] gives the n period moving average"

BorderTotals::usage = "BorderTotals[x_List] gives the border sums of x using TensorRank.\nBorderTotals[x_, Eliminate -> {n_Integer, m___Integer}, hd_:Plus, opts___] 'eliminates' dimensions n to m from the tensor, default by applying Plus.\nBorderTotals[x_, Partition -> y_List, hd_:Plus, opts___] partitions the border sums according to y = {{imin1, imax1}, {imin2, imax2},...} where the imaxj +1 = imin(j+1). A single number i may be used instead of {i=iminj, i=imaxj}. E.g. {1, {2, 4}, 5}. Per part, the complement is eliminated.\nUse x = Transpose[orig, {....}] to get the dimensions of x in consecutive order.\nNote: Eliminate and Partition are pseudo-options: must be mentioned as is and cannot be set by SetOptions.\nNote: the routine employs Apply[hd, x, {n, m}-1, opts]. Adding 1 at the input level agrees with Transpose and Permutation."

System`Partition::usage = Cool`Context`Key[AddedUsage, Partition, "Option Keyword for BorderTotals(Q). Partition -> y_List, partitions the border sums according to y = {{imin1, imax1}, {imin2, imax2},...} where the imaxj +1 = imin(j+1). A single number i may be used instead of {i=iminj, i=imaxj}. E.g. {1, {2, 4}, 5}. Per part, the complement is eliminated (default by Plus)."] (* loading order problems with Cool`Context` ?*)


BorderTotalsQ::usage = "BorderTotalsQ[tr_Integer, Partition -> y_List] provides merely a check on the partition. It is True if the partition is in right order and fully exhausts the tensor rank tr."

CasesMatch::usage = "CasesMatch[k_List, m_] uses SmatchQ"

Cumulate::usage = "Cumulate[x] cumulates the entries in the list x"

DataFilter::usage = "DataFilter[x__], for x a vector or a sequence of equal vectors, deletes equal positions where there are missing data. Default option is Symbol -> Indeterminate." 

DeleteElement::usage = "DeleteElement[l_List, e_, normal_:True] normally deletes e from the list, otherwise maps this deletion over the various occurences of e"

DeleteVectorPosition::usage = "DeleteVectorPosition[x_List, n_Integer]\n deletes position n in any vector in the list"

Dif::usage = "Dif[x_List] gives x - Lag[x], and thus takes the first difference.\nDif[x, n] takes the n-th difference."

DimensionMap::usage = "DimensionMap[x_, onlyLists_:True] similar to mapping Dimensions on x. By default only Lists are recognised."

DimensionNest::usage = "DimensionNest[x_, onlyLists_:True, sor_:True] repeated application of DimensionMap"

DimensionForm::usage = "DimensionForm[x_, onlyLists_:True, sor_:True], prints Nested DimensionMap in TableForm (though for vectors transposed)"

Distance::usage = "If f is a distance measure defined on two vectors (for example defined by using Inner), then Distance[f] can be used to generate a matrix of distances between various points. The default distance measure is NRadius, with the Euclidean distance. The definitions have been:
\n      Distance[f:NRadius][x_List, y_List] := f[x, y]  (*complicated, but for consistency*)
\n      Distance[f:NRadius][x__List] := Outer[f, {x}, {x}, 1]
\n      Distance[f:NRadius][{x__List}] := Outer[f, {x}, {x}, 1].
\n      For example: Distance[][{1, 2}, {3, 4}, {5, 6}]
\n      Also, Distance[Right][x] gives the rectilinear distance (Manhatten grid)
\n\nDistance[Min, m_List, w_List:{}] identifies the point with the minimal total distance to the other points. Here, m is a distance matrix m[i, j] with origin i and destination j, and w are weights (default set to units}. The routine thus determines the minimal column sum of (diag[w] . m)"

ExistQ::usage = "ExistQ[list, crit, levelspec_:{1}, opts___Rule] gives True if list is a nonempty List and some of its elements cause crit to be True. The routine uses MapLevel. Levelspec (default 1) and opts are for Level. Use levelspec Infinity when crit is to apply to any level.  Alternative to Select, Cases & Count. Note: a Hintikka observation is that the existential quantifier always requires a range, and therefor ExistQ depends on something, here a list. See AllQ."

FromMold::usage = "FromMold[mold_List, y_List:{}] supplements y with a list-structure comparable to mold as much as possible. Indeterminate's are used as placeholders. If y has an Indeterminate where mold has a list, then this is replaced too"

IndexedSort::usage = "IndexedSort[x_List] sorts x and supplies the list of 
indices of the original position. IndexedSort[x, p] sorts using the ordering function p. Note that the actual sort is on the pairs {x1, 1}, {x2, 2}, ..."

InsideTable::usage = "For simultaneous selection of one keyword from a set of alternative lists of rules, and TableForm presentation under standard headings.\nIt is assumed that:\n   Entry[Label1, Label2, ...] =\n     {Key1 -> value1, Key2 -> value2, ...}\nand thus gives a list of rules on keywords. (The labels classify the genesis.)\n1) InsideTable[Set, Entry, labels] fills the memory with Entry and those labels = {Label1, Label2, ...}. The head 'Entry' is used to find the various lists of rules. The label input can be the TableForm rule TableHeadings -> labels, or just labels compatible with such use. Each Set destroys earlier settings.\n2) InsideTable[Show, Keyi_] shows Keyi in TableForm under the earlier provided labels.\n3) InsideTable[head_, labels_] creates the inside of a table of entries head[..,..,...] by applying Outer on the labels, where head need not be defined."

UnitIndex::usage = "UnitIndex[x__] gives RateOfChange[x] + 1. Applies to lists as well, or primarily so, and uses Division. See Cool[Chaindex] for chained indices."

JoinNews::usage = "JoinNews[a_List, b_List] := Join[a, Complement[b,a]]"

Lag::usage = "Lag[x_List] gives the lagged list (Indeterminate prepended, last element dropped). Lag[x, n] moves elements n places to the right.\nLag[x_String] as in Lag[\"var\"] can be used as a formal representation for lagged values. Strings are used here, so that replacing as in ({Lag[\"var\"], var} /. var -> 100) has the proper effect"

Lead::usage = "Lead[x_List] gives the lead list (Indeterminate appended, first element dropped). Lead[x, n] moves elements n places to the left."

MapThreadReplace::usage = "MapThreadReplace[x_List, y_List] :=\n MapThread[ReplacePart[#1,#2, 1]&,  {x,y}]\nreplaces the first positions of the lists in x with the elements in y"

Matrix::usage = "Keyword"

MatrixRowSums::usage = "MatrixRowSums[matrix_List, borders_:False]\n adds the elements per row;  if borders===True then the first row and column of the matrix are excluded (these can be labels)"

MatrixTimesDiagonal::usage = "MatrixTimesDiagonal[matrix_List, diagonal_List, borders_:False] \nmultiplies a twodimensional matrix with a diagonal matrix formed from vector diagonal; if borders===True then the first row and column of the matrix are excluded (so that the vector diagonal must have one element less than the matrix row dimension)"

MoveToFirst::usage = "MoveToFirst[x_List, y_] moves element y to first position in list x"

NoNulls::usage = "NoNulls[x_List] removes Null elements from x\nNoNulls[z_][x_] gives If[Rest[x] === z, {First[x],Null}, x]"

NRadius::usage = "NRadius[x, c] gives the radius of point x with center c. If c is a number, then the point {c, c, c, ...} is taken. The default c = 0. Note: the name NRadius chosen to prevent conflict with Radius in Mathematica packages"

OnlyNulls::usage = "OnlyNulls[len_] gives  Table[Null, {i, len}]"


NonZero::usage = "NonZero[x_] deletes 0 and 0. from x without changing the structure of x (that can be lists). Only when x = 0 or 0., then output is Null. Test on nonzerohood by NonZeroQ[x] :=  x === NonZero[x]."

NonZeroQ::usage = "NonZeroQ[x_] :=   x === NonZero[x]."

NullQ::usage = "NullQ[x_List] gives a warning when x contains Null somewhere."

PartOnList::usage = "PartOnList[matrix_List, position_List] generalizes Part"

Places::usage = "Places[var_List,value_, apply_:List] a generalized version of Position[var, value] so that value may be a list of values. Also {{x___}} is set to {x}. If apply is Union, then the results are united, so that the positions are not related to separate value[i]."

PartOnList::usage = "PartOnList[matrix_List, position_List] generalizes Part"

Pick::usage = "Pick is a variant of Take, thought to be useful for data vectors.\nPick[x__List] takes from positions set by Options[Pick], default Take -> {1, Infinity}. The latter means that the length of the smallest list is taken.\nPick[i (,j)][x__List] maps Take[#, i]& or Take[#, {i, j}]& over {x}, similarly taking account of the minimal length in {x}"


Polygamy::usage = "Polygamy[x_list] finds those pairs in x that have the same first element. E.g. {{jan, marie}, {klaas, marie}, {jan, suze}}"

Prima::usage = "Prima[x] := Print[MatrixForm[x]] output facility"

UnitChange::usage = "No longer in use, see UnitIndex"

RateOfChange::usage = "1) RateOfChange[x, y] gives x / y - 1, while missing elements represented by Indeterminate generate Indeterminate again.\n2) RateOfChange[x] gives RateOfChange[x, Lag[x]]\n3) RateOfChange[Drop, x (,y)]  drops the resultant first element.\n4) RateOfChange[Range, x_List] gives of the rates of change the mininum, maximum, average, and the geometric mean growth from the first to the last element. \n5)RateOfChange[Lag, n, x] gives the geometric mean rate of change of x with respect to Lag[x, n]\nVariables x and y are tested with VectorQ"

Repeat::usage = "Repeat[command, x] = FoldList[command, First[x], Rest[x]]. Command values are: Plus for Cumulation, Times for multiplying and Divide for dividing throughout."

Smooth::usage = "Smooth[y_List, f:.75] develops z[t] = f z[t-1] + (1 - f) y[[t]], with z[1] = y[[1]]. The default f = 0.75 is a good smoother.\nSmooth[r][y_List, f:.75] develops z[t] = (1 + r)(f z[t-1] + (1 - f) y[[t]]).\nSmooth[Find][x_List, a_List:{}, b_List:{}, opts] finds the parameters f[1] and c that generate the best exponential forecast series for x. The forecast series starts with f[1] and the next forecast is the exponential smoother f[2] = c f[1] + (1 - c) x[[1]]. Etcetera. Numerical estimation is used, since the symbolic approach appears very slow. List a gives the estimation start values for f[1], and list b gives the estimation startvalues for the exponential update parameter c. So a = {f0, f1} and b = {c1, c2}. Empty lists use default values. Since the startvalue f[1] may be less relevant for long lists, you may enter a = {Set, f[1]} or a = {Set, Automatic}, and in the latter case f[1] = Average[x]. Options are passed on to FindMinimum. Results are: List -> the smoothed series, First -> f[1], Factor -> the smoothing factor c, Last -> forecast for the period beyond the list, Min -> SSE"

StDev::usage = "StDev[x] gives the maximum likelihood standard deviation of vector x, using Average and NRadius."

SymbolArray::usage = "SymbolArray[k_, n__] makes an array of Symbols concatenated of k and the index. See Array for possible specifications of n."

SymbolArrayRule::usage = "SymbolArrayRule[items_List, symbol_Symbol] makes a list of rules, assigning items to the symbol array"

TakeElements::usage = "TakeElements[l_, p_List] \n takes the elements of l from positions which have been found by Position[l,p]\nuseful for parallel structures:\ng = { w, {y,e,w}}\ng1 = { w1, {y1,e1,w1}}\nf = Position[g, w]\nTakeElements[g1, f]"


ToPolar::usage = "ToPolar[a_List, opts] gives {r, sequence of angles}. The angles are taken for the projections on planes. For example, with a = {x, y, z} then the angle in the {x, y} plane is arctan[y/x]. Option Method -> Complex is default, which means that a complex notation is used. Alternative settings are ArcTan and {ArcTan, Symbol}"

FromPolar::usage = "FromPolar[a_List, opts] for a = {r, sequence of angles} gives the point in Cartesian co-ordinates. See ToPolar, also for the options"

TensorForm::usage = "TensorForm[x_, opts___] applies TableForm, though for Rank 1 with TableDirection -> Row, i.e. is horizontal instead of vertical."

TransposeTensor::usage = "TransposeTensor[x, y___] doesn't fail to transpose vectors too, e.g. {1,1}"

Transposant::usage = "Transposant[level_Integer, len_Integer] gives the permutation (the second argument of Transpose), when the total number of levels is len, and {1, level} must be transposed"

TransposeToFirst::usage = "TransposeToFirst[x_List, y_List] transposes x, with the dimensions mentioned in y put on top (in that order)."

ToMatrix::usage = "ToMatrix[y] or ToMatrix[y, number] transforms a list of unequally sized lists into a matrix, by appending Indeterminate's. If no number is specified, then the maximum length of these lists is taken. Also, a single vector is turned into a matrix." 

TwoPointsToLine::usage = "f[t_] = TwoPointsToLine[t_Symbol, x_List, y_List, type_:Automatic] gives the line f[t] = t x + (1 - t) y for two arbitrary points x and y in n-space. Use ParametricPlot (3D) for plots. However, If type = Automatic and n = 2, then the more standard representation f[t] = m t + c is given, for which one uses Plot"

Begin["`Private`"]

Angle[a_List, b_List] := ArcCos[a . b / Sqrt[ (a . a) * (b . b) ]]
Angle[a_List] := ArcTan[Last[a] / Sqrt[ Add[Drop[a, -1]^2] ]]

AggregatorQ[ag_] := AllQ[ag, ((# === 0)||(# === 1))&, {2}] &&
	(Map[Add,#] === Table[1, {Length[#]}])&[TransposeTensor[ag]]

Aggregate[ag_, y_, spec_:{1}] := 	
	Module[{level =First[spec], dag =Dimensions[ag], dy = Dimensions[y], tp},
	If[Last[dag] =!= dy[[level]], Return[{"Dimensions dont fit",
			Aggregate -> dag, Table -> dy , Level -> level}]];
	If[!AggregatorQ[ag], Return[{ag," is not an aggregator"}]];
	tp = Transposant[level, Length[dy]];
	TransposeTensor[ag . TransposeTensor[y, tp], tp]
	](*end module*)

Applist[x_, y_] := If[ListQ[y], x @@ y, x[y]]

ArgMaxQ[x_, i_] := (x[[i]] == Applist[Max, x])

ArgMinQ[x_, i_] := (x[[i]] == Applist[Min, x])

Average[x_List] := (Plus @@ x) / Length[x]

Average[x_List, n_Integer] :=

	Average /@ Table[Take[x, {i, i+n-1}], {i, 1, Length[x]-n+1}]


BorderTotals[x_List] :=

	Module[{a,b,d},
	
		d = TensorRank[x];
		a = Range[d];
		b = NestList[ RotateRight, a, d-1];
		If[d ===1, d={1}];
		Map[Apply[ Plus, Transpose[x,#], d-1]&,   b]

	] (* end module BorderTotals, part 1 *)

BorderTotals[x_List, Eliminate -> {n_Integer, m___Integer}, hd_:Plus, opts___] := 
	Apply[hd, x, {n, Max[n,m]}-1, opts]
			
BorderTotals[x_List, Partition -> y_List, hd_:Plus, opts___] :=

	Map[BorderTotals[TransposeToFirst[x, #], 
	Eliminate -> {Length[#] + 1, TensorRank[x]}, hd, opts]&, 
				Map[Flatten[{#}]&, y] (*warrants inputs of lists*)]


BorderTotalsQ::false = "Partition -> `1` does not exhaust tensor rank `2`"

BorderTotalsQ[tr_Integer, Partition -> y_List] := 
	Apply[Join, Map[If[ListQ[#], Apply[Range, #], {#}]&, y]] === Range[tr]

CasesMatch[k_List, m_] := TakeElements[k, Position[SmatchQ[k,m], True]]

Cumulate[x_] := FoldList[Plus, First[x], Rest[x]]

DeleteElement[l_List, e_, normal_:True] := 

	If[normal, Delete[l, Position[l,e]], Map[Delete[l,#]&, Position[l,e]]]

DeleteVectorPosition[x_List, n_Integer] :=

	If[VectorQ[x], Delete[x, n], Map[DeleteVectorPosition[#,n]&, x]]

Options[DataFilter] = {Symbol -> Indeterminate}

DataFilter[x__] := Module[{pos, z, temp, symb},
		symb = Symbol /. Options[DataFilter];
		z = Transpose[{x}];
		pos = Union[Map[First, Position[z, symb]]];
		If[pos === {}, {x},
			temp = DeleteVectorPosition[{x}, pos[[1]]];
			DataFilter @@ temp]]

Dif[x_List] := x - Lag[x] 

Dif[x_List, n_] := Nest[Dif, x, n]  

DimensionForm[x_, onlyLists_:True, sor_:True] := 
		Module[{y}, y = DimensionNest[x, onlyLists, sor]; 
		TableForm[If[VectorQ[y], {y}, y]]] (*end module*)

DimensionNest[x_, onlyLists_:True, sor_:True] := 

	If[sor, 
	Sort[Rest[NestList[DimensionMap[#, onlyLists]&, x, Depth[x]-1]],
	LessComplicatedQ],
	Rest[NestList[DimensionMap[#, onlyLists]&, x, Depth[x]-1]]]

DimensionMap[x_, onlyLists_:True] := If[onlyLists && !ListQ[x], 1, 
	If[Depth[x]===2, Length[x], Map[DimensionMap[#, onlyLists]&,x]]]

Distance[Min, mat_List, x_List:{}] :=
	Module[{c, s, m, w},
	w = If[x === {}, Table[1, {i, Length[mat]}], x]; 
	c = mat w; s = Add /@ Transpose[c]; m = Min[s];
	{Distance -> mat, List -> w,
	Matrix -> c, Sum -> s, Min -> m,
	Position -> Position[s, m][[1, 1]]}	]

Distance[Right][x__] := Distance[Add[Abs[#1 - #2]]&][x]
Distance[f_:NRadius][x_List, y_List] := f[x, y]    
Distance[f_:NRadius][x__List] := Outer[f, {x}, {x}, 1]
Distance[f_:NRadius][{x__List}] := Outer[f, {x}, {x}, 1]


FromMold[][mold_, y_] :=
	Module[{m, n, a = y, lqa, lqm, f, sendonq},
	If[And[!ListQ[a], a === Indeterminate, ListQ[mold]], a = {}];
	If[!ListQ[a], a,
		n = Length[y]; 	m = Length[mold]; 
		If[m > n, a = Join[a, Table[Indeterminate, {m - n}]]];
		lqm = ListQ /@ mold;
		lqa = ListQ /@ a;
		pos = Position[Thread[Implies[lqm, lqa]], False];
		If[pos =!= {}, a = ReplacePart[a, {}, pos]]; 
		sendonq = Or @@ Join[lqm, {pos =!= {}}];
		If[!sendonq, a, 
	  		a = Thread[f[mold, a]] /. f -> FromMold[]]
		] (*endif listq*)
	](*end module*)
	
FromMold[mold_List, y_:{}] := FromMold[][mold, y]


IndexedSort[x_List] := Transpose[Sort[Transpose[{x, Range[Length[x]]}]]]

IndexedSort[x_List, p_] := 
	Transpose[Sort[Transpose[{x, Range[Length[x]]}], p]]

InsideTable[see_, hd_] := 
		 Outer[see, Apply[Sequence, If[RuleQ[hd], Last[hd], hd]]]

InsideTable[Set, result_, hd_] := 
		(Set[InsideTable[TableHeadings], If[RuleQ[hd], Last[hd], hd]];
		Set[InsideTable[Table], InsideTable[result, hd]];)

InsideTable[Show, item_] := TableForm[item /. InsideTable[Table],
			TableHeadings -> InsideTable[TableHeadings]]

JoinNews[a_List, b_List] := Join[a, Complement[b, a]]


AllQ[x_, test_, levelspec_:{1}, opts___Rule] :=   (x =!= {}) &&
	ListQ[x] && Apply[And, MapLevel[test, x, levelspec, opts]]

ExistQ[x_, test_, levelspec_:{1}, opts___Rule] :=  (x =!= {}) &&
	ListQ[x] && Apply[Or, MapLevel[test, x, levelspec, opts]]


MapThreadReplace[x_List, y_List] := MapThread[ReplacePart[#1,#2, 1]&,  {x,y}]

MatrixRowSums[matrix_List, borders_:False] :=

	If[borders === False,  Map[Apply[Plus, # ]&,  matrix],
	
	Prepend[  MatrixRowSums[Rest[matrix]] /.  (x_ + y_ -> {y, x}), {"item", "total"}]
		
	] (* end if MatrixRowSums; see MatrixQ *)


MatrixTimesDiagonal[matrix_List, diagonal_List, borders_:False] :=

	If[borders === False,  Transpose[Transpose[matrix] diagonal ],

	Module[{headline = matrix[[1]], m},
		(* first row and column excluded, e.g. are textual labels *)
		newborder = Prepend[diagonal, Automatic];
		m = MatrixTimesDiagonal[Rest[matrix], newborder];
		Prepend[m, headline]  /. Automatic x_ -> x
	] (*end module*)]



MoveToFirst[x_List, y_] := Prepend[Delete[x, Position[x, y][[1,1]]], y]


Lag[x_List] := Prepend[Drop[x, -1], Indeterminate]

Lag[x_List, n_] := Nest[Lag, x, n]

Lead[x_List] := Append[Rest[x], Indeterminate]

Lead[x_List, n_] := Nest[Lead, x, n]

NoNulls[z_][x_] :=   If[Rest[x] === z, {First[x],Null}, x]

NoNulls[x_List] := ReleaseHold[x /. Null -> Hold[Sequence[]]]

OnlyNulls[len_] :=  Table[Null, {i, len}]


NonZero[x_] := If[ListQ[x], DeleteElement[DeleteElement[x, 0.], 0],
					If[Sign[x] === 0, Null, x]]

NonZeroQ[x_] := (x === NonZero[x])

NullQ::true = "Warning: list contains Null, use Indeterminate to represent missing data"

NullQ[x_List] := If[MemberQ[Flatten[x], Null], Message[NullQ::true]]


NRadius::len = "The lists are of unequal length"

NRadius[x_List, c_List] := 
	If[Length[x] =!= Length[c], Message[NRadius::len],
				Sqrt[Plus @@ ((x - c)^2)]]

NRadius[x_List, c_:0] := Sqrt[Plus @@ ((x - c)^2)]


PartOnList[matrix_List, position_List] :=

	If[Depth[position] =!= 2, Map[PartOnList[matrix,#]&,position],

	Apply[Part, Join[ {matrix},position]] ]


Pick[i_?((IntegerQ[#]||(Abs[#] === Infinity))&)][x__List] := 
	Module[{t}, t = Abs[i];
	t = Min @@ Prepend[Length /@ {x}, t];
	Take[#, Sign[i] t]& /@ {x}]


Pick[i_?IntegerQ, 
	j_?((IntegerQ[#]||(# === Infinity))&)][x__List] := 
	Module[{t},
	t = Min @@ Prepend[Length /@ {x}, Abs[j]];
	Take[#, {Min[Abs[i], t], t}]& /@ {x}]
	
Options[Pick] = {Take -> {1, Infinity}};

Pick[x__List, opts___Rule] := 
	(Pick @@ (Flatten[{Take} /. {opts} /. Options[Pick]]))[x]




Polygamy[data_List] := Module[{sd, f, f2, qt}, 
	If[!MatrixQ[data], Return[{}]];
	sd = Sort[data]; 
	(*quick test*)
	f = (First /@ sd); f2 = Union[f];
	If[ Length[f] === Length[f2], Return[{}]];
	(*Apparently, there are more occurences of the same label in the
	first position of the pairs*)
	qt = Transpose[{Drop[sd, -1], Rest[sd]}];
	qt = If[First[First[#]] === First[Last[#]], #, Sequence[]]& /@  qt;
	Union[Flatten[NoNulls[qt], 1]]	
	] (*end module*)


Prima[x_] := x // MatrixForm // Print



Places[var_List, value_, apply_:List] :=  

	If[!ListQ[value],
	Position[var, value],
	Apply[apply, Map[Position[var, #]&, value]]]  /. {{x___}} -> {x}

Repeat[command_, x_] := FoldList[command, First[x], Rest[x]]


SymbolArray[k_, n__] :=
	Module[{ds, d, x}, ds = Array[d, n];
	ds /. d[x__] :> ToExpression[ToString[k] <>  StringJoin[Map[ToString, {x}]]]]

SymbolArrayRule[items_List, symb_Symbol] := 
		Module[{r}, 
		Off[General::spell1];
		r = Thread[Rule[items, SymbolArray[symb, Length[items]] ]];
		On[General::spell1];
		r] (*end module*)


TakeElements[l_, p_List] := Table[Apply[Part, Join[{l},Part[p,i]]],{i,Length[p]}]


ToMatrix[y_] := ToMatrix[y, Max[ Length /@ y] ]

ToMatrix[y_, len_] := If[VectorQ[y], {y},
	If[MatrixQ[y] || !NumberQ[len], y, 
	Join[#, Table[Indeterminate, {len - Length[#]}]]& /@ y]] (*end if*)

(* different treatment depending upon input format *)
Options[ToPolar] = {Method -> Complex}

ToPolar[a_List, opts___Rule] := 
	Module[{x, y}, x = Drop[a, -1]; y = Rest[a];
	{NRadius[a]} ~Join~ 
	Switch[
	Method /. {opts} /. Options[ToPolar],
	{ArcTan, Symbol}, ArcTan /@ (y/x),
	ArcTan, at /@ Transpose[{x,y}],
	_, Arg[x + I y] ]]

at[{x_, y_}] := Switch[Sign[x], 0, Sign[y] Pi/2, 1, ArcTan[y/x],
	-1, If[Sign[y] === 0, Pi, Sign[y] Pi - ArcTan[y/ Abs[x]]]]


FromPolar[a_List, opts___Rule] :=
	Switch[
	Method /. {opts} /. Options[ToPolar],
	{ArcTan, Symbol}, FromPolar1[a],
	ArcTan, FromPolar1[a],
	_, FromPolar2[a]]

FromPolar1[a_List] :=
	Module[{b},
	b = Reverse[FoldList[(#1/#2)&,1, Tan /@ Reverse[Rest[a]]]];
	a[[1]] b / Sqrt[b . b]]


ArgInverseRules = 
	{Cos[Arg[a_ + b_ I]] :> a / Sqrt[a^2 + b^2], 
	Sin[Arg[a_ + b_ I]] :> b / Sqrt[a^2 + b^2]};

FromPolar2[a_List] := 
  Module[{var, phi, l, Global`x, r, xs, rs, 
  	eqre, eqr, eqs, s, ms, pos, xv, yv, cossy, sinny}, 
    var = Length[a]; phi = Rest[a]; 
    xs = SymbolArray[Global`x, var]; 
    xv = Drop[xs, -1] ;
    yv = Rest[xs];
    rs = xv^2 + yv^2;
    (*Solve deals badly with Sqrt, so take Squares, here, 
    and eliminate solutions that do not have the proper Sign*)
    cossy = (Cos[phi] /. ArgInverseRules);
    sinny = (Sin[phi] /. ArgInverseRules);
    l = (Sign /@ cossy) ~Join~ {Sign[Last[sinny]]};
    eqre = Thread[xv^2 == rs cossy^2];
    eqr = First[a]^2 == xs . xs; 
    eqs = Join[eqre,{eqr}]; 
    s = Solve[eqs, xs]; 
    If[s === {}, Return[eqs], s = xs /. s];
    If[(Head[First[l]] =!= Integer) || !AllQ[Flatten[s], NumberQ],
    	(*algebraic*) Union[Abs[s]] /. Abs -> Identity, 
	(*numeric*)
    	ms = Map[Sign, s, {2}];
		pos = Position[ms, l];
		TakeElements[s, pos][[1]]]]


(*protect to numbers, otherwise long and slow symbolics*)

errorf[x_List][a_?NumberQ, b_?NumberQ] := 
					(lis = Smooth[Prepend[x, a], b];
					Plus @@ ((x - Drop[lis, -1])^2))

Smooth[Find][x_List?VectorQ, a0_List:{}, b0_List:{}, opts___Rule] :=
	Module[{a, b, av, alis, blis, res, f},
		av = Average[x];
		alis = If[a0 === {}, {a, av/2, av}, Prepend[a0, a]];
		blis = If[b0 === {}, {b, .25, .75}, Prepend[b0, b]];
		res = FindMinimum @@ 
		(If[a0 =!= {} && First[a0] === Set,
				a = If[Last[a0] === Automatic, av, Last[a0]];
				{errorf[x][a, b], blis, opts}, 
				(*else*)
				{errorf[x][a, b], alis, blis, opts}]);
		Results[Smooth] = res ;
		errorf[x][a, b] /. Last[res]; (*update lis*)
		{Last -> Last[lis], First -> a, Factor -> b,
		List -> lis, Min -> First[res]} /. Last[res]]


Smooth[t_][y_List, x_:0.75] := 
	FoldList[((1+t)(x*#1 + (1 - x)*#2))& , First[y], Rest[y]]

Smooth[y_List, x_:.75] := FoldList[(x #1 + (1-x) #2)&,  First[y], Rest[y]]

TensorForm[x_, opts___] := TableForm[ If[TensorRank[x]===1, {x}, x], opts]
	(* does not work: TableForm[x, 
	TableDirections -> If[TensorRank[x]===1, Row, Automatic], opts] *)

TransposeTensor[x_, y___] := 
		If[TensorRank[x] === 1, Map[List, x],  Transpose[x, y]]

Transposant[level_Integer, len_Integer] := 
Fold[rp, Table[i, {i, len}], {s[1, level], s[level, 1]}] /. {s ->Sequence, rp -> ReplacePart}

TransposeToFirst[x_List, y_List]  := 
	Module[{tr, ta, com, n, f},
		tr = TensorRank[x]; ta = Table[i, {i, tr}]; 
		com = Complement[ta, y];  n = Length[y];
		MapThread[ (f[#1] = #2)&,  {y, Take[ta, n]}];
		MapThread[ (f[#1] = #2)&,  {com, Take[ta, n-tr]}];
		Transpose[x, Map[f, ta]]
	] (* end Module *)

TwoPointsToLine::len = "Unequal lengths `1` and `2`";

TwoPointsToLine[t_Symbol, x_List, y_List, type_:Automatic] := 
		Module[{n, m}, n = Length[x]; m = Length[y];
		If[n =!= m, Message[TwoPointsToLine::len, n, m];Return[]];
		If[(type =!= Automatic) && (n == 2), n = 3];
		Switch[n,
		0, 0,
		1, t,
		2, Expand[((y[[2]]-x[[2]]) t + 
	         (x[[2]] y[[1]] - x[[1]] y[[2]]))/ (y[[1]] - x[[1]])],
		_, t x + (1 - t) y]
		] (*end Module*)



UnitIndex[Drop, x__] :=  Drop[UnitIndex[x], 1]

UnitIndex[x_?VectorQ] := UnitIndex[x, Lag[x]]

UnitIndex[x_?VectorQ, y_?VectorQ] := 
		Module[{r, mesq},
		(* because of change of indicator of missing data*)
		NullQ[x]; NullQ[y]; 
		mesq = Messages /. Options[Division];
		If[mesq === Off, Off[Power::infy]; Off[Infinity::indet]];
		r = x / y;
		If[mesq === Off, On[Power::infy]; On[Infinity::indet]];
		r = r /. {Infinity -> Indeterminate, 
			ComplexInfinity -> Indeterminate }
		] (*end module*)

UnitIndex[x_, y_] :=  Division[x, y]


RateOfChange[Range, a_List?VectorQ] :=
	Module[{z}, z = RateOfChange[Drop, a];
	{Min -> Min[z], 
	True -> (Last[a] / First[a])^(1/(Length[a]-1)) - 1, 
	Average -> Average[z], Max -> Max[z]}]

RateOfChange[Drop, x__] :=  Drop[RateOfChange[x], 1]

RateOfChange[x_List?VectorQ, y_List?VectorQ] :=  (UnitIndex[x, y] - 1)

RateOfChange[x_, y_] :=  (Division[x, y] - 1)

RateOfChange[x_] := RateOfChange[x, Lag[x]]

RateOfChange[Lag, n_Integer, x_List?VectorQ] :=  
		UnitIndex[x, Lag[x, n]]^(1/n) - 1




StDev[x_?VectorQ] := NRadius[x, Average[x]] / Sqrt[Length[x]]

(*here VectorQ, since Length only takes the next highest level *)

End[]

AddedUsage[{Eliminate}]

EndPackage[]
