(*********************************************************************

	Cool`Manager`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1993/95 Thomas Cool *)
(* :Package Version: 1.03 Thomas Cool, July 1995.*)
(* :Mathematica Version: 2.2.2 *)
(* :Keywords:     Input, Output, Options, Results *)
(* :Limitation: *)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)


BeginPackage["Cool`Manager`", {"Cool`List`", "Cool`Common`"}]

CurrentOptions::usage = "CurrentOptions[command_Symbol, {news___Rule}, opts___] applies ReplaceRule to Options[Command] and the news. The values within Options[command] are not affected (like with SetOptions)."


Dump::usage = "Dump[x] a generic place for routines to put surplus information, (e.g. for error checking)."

OptionsFilter::usage = "OptionsFilter[symbol, options___] returns a sequence of those options that are valid options for symbol. The options may be interchanged with lists of options. (Adapted from Utilities`FilterOptions`.) "

FilterSetOptions::usage = "FilterSetOptions[symbol, options...] sets the options after first filtering out the irrelevant ones."

Options[HoldShell] = Sort[{ReleaseHold -> True, SetResults -> False, Union -> True, Select -> Automatic, ReEvaluate -> True}];

HoldShell::usage = "HoldShell[f_, expr_Hold, opts___Rule] allows the selection of an earlier result from Results[f] rather than doing ReleaseHold[expr]. The following default Options[HoldShell] apply:\nReEvaluate -> False: override options ReleaseHold and SetResults, prevent reevaluation whenever possible, use all earlier Results whenever possible, do ReleaseHold only when the Results[f] give a Null answer, and store such new results.\n   (The following two options are relevant when ReEvaluate -> True:)\nReleaseHold -> True: do not use Results;\nSetResults -> False: do not update Results[f];\nSelect -> Automatic: select Results keywords from expr. Expr then has the format Hold[{key -> g[y], ...}], i.e. a list of rules within Hold. Alternatively, Select -> {key, ...} explicitly, and then the only requirement may be that the evaluation of expr generates a list of rules {key -> x, ...} such that Result[f] might be updated;\nUnion -> True: passed on to RuleReplace.";

Kilroy::usage = "Debugging tool to temporarily put into loops. Kilroy[x] messages and prints x. If x is Blank, then the Kilroy statement."

MemoryConstrainedQ::usage = "MemoryConstrainedQ[proc_, limit_Integer] \n execute proc, and return True if the memory constraint applied"

Redefine::usage = "Redefine[x_, old_List, new_List ] Switches x from old to new. Normally x is an element of old, but when x is a list, then x[[1]] is taken as the Switch expression while the other elements are part of the Switch body"

ReEvaluate::usage = "Keyword may indicate whether routines should re-evaluate functions and options, or that they can take existing values. See HoldShell."

ReEvaluate::rec = "ReEvaluate -> True may be slow for recursive calls from `1`";

ResetOptions::usage = "ResetOptions[x] performs Options[x] = ToExpression[\"Options$\" <> ToString[x]], thus sets the options to a predefined value, presumably given at startup in a package or saved inbetween."

Results::usage = "Results[f] is a list of rules to store results of keyword f. Routines may call on Results instead of recomputing results. See SetResults."

ResetResults::usage = "ResetResults[f] resets Results[f] to either {} or a predefined Results$f."

SetResults::usage = "SetResults[f, {forf___Rule}, opts___Rule] sets options in Results[f] to new values supplied in forf. The opts are directed to ReplaceRule. If Clear -> True is given, with or without forf-rules, then there is a ResetResults (first)."


Begin["`Private`"]


CurrentOptions[command_Symbol, x_List, opts___] :=
	
	If[x === {}, Options[command], ReplaceRule[Options[command], x, opts]]


OptionsFilter[command_Symbol, opts___ ] :=
	Module[{keywords = First /@ Options[command]},
	Sequence @@ Select[ Flatten[{opts}], MemberQ[keywords, First[#]]& ]]

FilterSetOptions[command_, opts___] :=
SetOptions[command, OptionsFilter[command, Sequence @@ Flatten[{opts}]]]


HoldShell::head = "Head should be Rule and is not: `1`";


HoldShell[f_?((!RuleQ[#])&), do_Hold, opts___Rule] :=
	Module[{redo, store, keys, reeval, out, hd, x, y, equals},
	{redo, store, keys, reeval} = {ReleaseHold,  SetResults, 
		Select, ReEvaluate} /. {opts} /.Options[HoldShell];
	If[!reeval, store = True];
	(*keys*)
	If[keys === Automatic, (*do has format Hold[{key -> g[x]}] *)
		hd = do[[1,1,0]];
		If[hd =!=Rule, Message[HoldShell::head, hd]];
		keys = do /. {(x_ -> y_) :> x}]; (*end automatic*)
	keys = Flatten[{ReleaseHold[keys]}];
	(*redo?*)
	If[(!redo) || (!reeval), 
		out = Thread[keys -> (keys /. Results[f])]; 
		If[reeval,Return[out],
			(*redo if key -> key, or key -> Null*)
			equals = Flatten[out /. ((x_ -> y_) :> {}    /; (x =!= y))];
			redo = (equals =!= {}) || (Position[keys/.out, Null] =!= {});
			If[!redo, Return[out]]]];
	If[redo, out = ReleaseHold[do];
		If[store, SetResults[f, out, opts]]; out]
	](*end module*)


Kilroy::here = "Kilroy was here: `1`";
Kilroy[x_:"Kilroy was here"] := (Message[Kilroy::here, x]; Print[x])


MemoryConstrainedQ[proc_, limit_Integer] :=

	Not[MemoryConstrained[proc, limit - MemoryInUse[], False]]



Redefine[x_, categories_List, new_List ] :=

	Apply[Switch, Flatten[{x, Transpose[{Flatten[categories], Flatten[new]}]}]]


ResetOptions[x_] := (Options[x] = ToExpression["Options$" <> ToString[x]])

ResetResults[f_] := Module[{dollar},
	dollar = ToExpression["Results$" <> ToString[f]];
	Results[f] = If[ListQ[dollar], dollar, {}] ] (*end module*)

SetResults[f_,{fopts___Rule}, opts___Rule] :=
	Module[{reset, dollar},
	reset = Clear /. {opts} /. Clear -> False;
	If[!ListQ[Results[f]] || (reset === True), ResetResults[f]];
	Results[f] = ReplaceRule[Results[f], {fopts}, opts]
	] (*end module*)

SetResults[f_?((!RuleQ[#])&), Clear -> x_] := If[x, ResetResults[f], Results[f]]

End[]

Protect[ReEvaluate]

EndPackage[]
