(*******************************************

	Cool`Queue`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1997 Thomas Cool *)
(* :Package Version: 1.04,  July 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function.
 *)

Needs["Cool`Common`"]

BeginPackage["Cool`Queue`", Join[{"Cool`Statistics`Common`", 
"Statistics`DescriptiveStatistics`", "Statistics`Common`DistributionsCommon`", 
"Statistics`DiscreteDistributions`", 
"Statistics`ContinuousDistributions`"}, $Cool]]

AdjustedRates::usage = "AdjustedRates[labda, muth, perc] adjusts rho for average loss days in the total of working days. A theoretical value for rho can be determined from the actual arrival rate labda and the theoretical service rate muth, but the proper values of the utilisation and service rates should include a proportion of loss. For example, if there are 2 servers, 360 working days per year, and a total of 10 loss days, then rho = labda/muth + 10 / (2 * 360), and then mu = labda / rho. Hence, in this case, choose perc = 10 / (2 * 360)" 

BerthOccupancyPlot::usage = "BerthOccupancyPlot[labda_:2 / 219, nships_:80, opts___Rule] uses SingleServer to plot a typical situation at a harbour. Assumed is that there are 80 ships of 100,000 Tonnes annually visiting a berth, giving labda = 80 Ship / (24 * 365 Hour) = 2 /219. The service or unloading time is given as 1 / mu = rho / labda. Two example results are:\n
\nrho =           60 %            30 %
\nunloading     5256 h          2628 h
\nwaiting       7884 h          1126 h
\ntotal        13140 h          3754 h
\nThe conclusion is that increasing the unloading capacity, at the cost of a lower utilisation rate, could significantly reduce harbour time per ship"


MeanArrivalRate::usage = "Keyword. Expected number of arrivals per unit of time (common: labda)"

MeanServiceRate::usage = "Keyword. Expected number of service departures per unit of time, of units completing service (common: mu)"

MeanUtilisationRate::usage = "Keyword. MeanArrivalRate / (NServers * MeanServiceRate per server) (common: rho)"

MeanNQueue::usage = "Keyword. Expected queue length (in units of some kind)  (common: Lq)"

MeanNSystem::usage = "Keyword. Expected number of units in the system = MeanNQueue + MeanNBeingServed  (common: L)"

MeanNBeingServed::usage = "Keyword. Expected number of units being served (common: L - Lq)"

MeanProcessTime::usage = "Keyword. Also known as the total wait time: the expected time per unit for the total process = MeanQueueWait + MeanServiceTime (common: W)"

MeanServiceTime::usage = "Keyword. Expected time at the servers, per unit (common: W - Wq)"


MeanArrivalTime::usage = "Keyword. 1 / MeanArrivalRate = the maximum to the MeanServiceTime = the average 'time per unit' of the whole system"

MeanQueueWait::usage = "Keyword. Expected waiting time in the queue, per unit (common: Wq)"

NQueue::usage = "Keyword. Number of units in the queue, dependent upon time"

NSystem::usage = "Keyword. Number of units in the system = NQueue + NBeingServed (common: En)"

NBeingServed::usage = "Keyword. Number of units being served"

NServers::usage = "Keyword. Number of Servers"

MultipleServers::usage = "MultipleServers[labda, mu, s]  gives the steady state queueing output for the system comprised of:\n(a) the inter-arrival rate has an independent exponential distribution with mean labda,\n(b) the servers are independent with each an exponential distribution with mean service time 1/mu.\nNote: No useful output is possible when the utilisation rate labda/(s mu) = rho > 1 (more arrivals than departures).\n\nNote: The routine also defines the probability that exactly n items are in line (queue and being served): \n \nPr[N, n], for n = 0,  0 < n <= s, and n > s \n \nThe probability regarding total process time t: \n \nPr[CDF, t]\nP{ProcessTime > t} = 1 - Pr[CDF, t]"

MultipleServersContour::usage = "MultipleServersContour[smin, smax, range, opts___Rule] plots the MultipleServers contours of s from smin to smax (integer values), for rho = labda/(s mu) in range (with default {0, 1}) and for y = mu Wq = mu MeanQueueWait. Options are passed on to Plot"




SingleServer::usage = "SingleServer[labda, mu, var_:Automatic] gives the steady state queueing output for the system comprised of:\n(a) the inter-arrival rate has an independent exponential distribution with mean labda,\n(b) the servers have a common distribution with mean service time 1/mu and variance var, and can be regarded as one server.\nNote: When var is not specified, then it automatically is assumed that the 'aggregate server' has an exponential service time, and hence var = 1/mu^2.\nNote: For a constant service rate, give var = 0.\nNote: With Poisson input and Erlang (gamma) service time, give var = 1 / (k mu^2) with k the number of service phases\nNote: No useful output is possible when the utilisation rate labda/mu = rho > 1 (more arrivals than departures).\n\nNote: When var = Automatic, i.e. an exponential service time, then the routine also defines:\n\nThe probability that exactly n items are in line (queue and being served):
\n
\nPr[N, 0] = 1 - rho
\nPr[N, n] = (1 - rho) rho^n
\n
\nThe probability regarding process time t:
\n
\nPr[PDF, t] = (mu - labda) E^((labda - mu) t)
\nPr[CDF, t] = 1 - E^((labda - mu) t)
\n
\n(so find: P{ProcessTime > t} = 1 - Pr[CDF, t] = E^((labda - mu) t) )";

SSFiniteSource::usage = "SSFiniteSource[labda, mu, M] gives the single server results when the number of customers in the source is exactly M"

SingleServerModel::usage = "SingleServerModel[{x___Rule}, elims___List, opts___Rule] is a SolveFrom application based on SingleServer (with the default variance 1/mu^2), that allows one to solve for the labda and/or mu given other system characteristics.\nE.g. SingleServerModel[{MeanNSystem -> 2, MeanArrivalRate -> 100/60}]"

MultipleServersModel::usage = "MultipleServersModel[{x___Rule}, elims___List, opts___Rule] is a SolveFrom application like SingleServerModel. It has only been added here for completeness, since it appears that Solve cannot really tackle this kind of model."

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
