(*******************************************

	Cool`Shipping`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1997 Thomas Cool *)
(* :Package Version: 1.03,  July 1998.*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function.*)
(* :Literature: P. Alderton, "Sea Transport", Reed 1995; 
	Wijnolst & Wergeland, "Shipping", Delft UP 1996*)

Needs["Cool`Common`"]

BeginPackage["Cool`Shipping`", 
Join[{"Cool`Money`", "Cool`Databank`", "Cool`Economic`Common`", "Cool`Freight`", "Cool`Physics`", "Cool`Time`", "Miscellaneous`Units`", "Miscellaneous`SIUnits`", "Cool`TScience`"}, $Cool]]

AgeEffectOnSpeed::usage = "AgeEffectOnSpeed[t] gives the reduction factor of the optimal speed as a function of the time since dry-dock, measured in months (Sea Transport, p75)"

CraneTime::usage = "Keyword to indicate the time that cranes are actively 
loading or unloading a ship, see TimeSheet.\nCraneTime[Sum][d] determines the duration of crane time (in days), if d is a TimeSheet object.\nCraneTime[][load, speed, n:1] gives the crane time for the load when n hatches have the same handling speed (per time unit). This is often seen as the theoretical lay time.\nCraneTime[Find][data_List, requiredHours_?NumberQ] runs through the TimeSheet object data list, cumulates the hours of crane time, and finds on which day the required hours have been completed"

CubeLaw::usage = "CubeLaw[speed, standard speed, standard fuel use] applies 
the cube law to derive fuel use at speed, by using the standard reference 
points. See JCurve";

Deadweight::usage = "Keyword for carrying capacity of ships (in Tonnes)"

Demurrage::usage = "Demurrage[p, ratio, a, u] gives the demurrage or despatch data for demurrage p ($/day), the Despatch/Demurrage ratio, available days a, and used days u. Note that available days can be found by a = load / handlingspeed (in days)"

Draft::usage = "Draft[dwt] gives the approximated draft in meters for a loaded ship with a deadweight of dwt (long) tons"

EquilibriumFreightRate::usage = "Keyword. EconomicSpeed == MostProfitableSpeed, Alderton p72"

EconomicSpeed::usage = "Keyword. Maximal profit per tonnekilometer, maximised over speed"

FuelPerDay::usage = "FuelPerDay[v, dwt] gives the fuel use per day in tons, 
for a speed of v knot, and a dwt in tons.
\nFuelPerDay[v, dwt, {c, b, a},{d, e}] gives a general formula for speed and dwt, with c, b, a the coefficients in a polynomial c v + b v^2 + a v^3, and (d dwt + e) the use at benchmark speed. Default parameter values are the same as JCurve's, e.g. enter {}'s.
\nApply the same transformations for km/h and ageing as for JCurve. Note that the optimal speed of the benchmark changes by implication (without further adjustment).
\nFuelPerDay[Find, v_Symbol, dwt, {c, b, a}, {d, e}, {pd, pv}, adj:1] gives 
the optimal use of fuel per day, using the {c, b, a} polynomial, the (d dwt + e) benchmark, the per day cost pd and fuel cost pv\nSet the adjustment factor for transformations like e.g. adj = 1/AgeEffectOnSpeed[t]\nIn a ballast sail, most ships go 10% faster at the same fuel consumption. Either use FuelPerDay[.90 v, dwt], or LoadEffectOnSpeed or FuelDraftReduction";

FuelDraftReduction::usage = "FuelDraftReduction[d, dwt, sd:Automatic] gives 
the reduction in fuel use in ton per day for a draft d, for a deadweight 
dwt in ton and a summer draft sd (used to measure the dwt). If sd==
Automatic, then Draft[dwt] is used for the default loaded summer draft\n
FuelDraftReduction[TPI, tpi, d, sd] gives the reduction in fuel use in tons 
per day, for a value of tons per inch tpi, draft d, and summer draft sd"

GT::usage = "GT[V] gives the gross tonnage where V is the total volume of all enclosed spaces in cubic meters"

JCurve::usage = "JCurve[v, c:.0486698, b:-.00479899, a:.000458904] uses a 
polynomial with these coefficients to derive the fuel use for v knot, RELATIVE to the use at some benchmark speed . For v km/hour give JCurve[v/1.852, ...] and for the effect of hull roughness use JCurve[v/AgeEffectOnSpeed[t], ...] (so ageing can be seen as an increase in virtual speed). Note that the coefficients change when the speed is measured in a different dimension or age";

LayTime::usage = "Keyword to indicate the time that the ship is not sailing, see TimeSheet.\nLayTime[Sum][d] determines the duration of lay time (in days), if d is a TimeSheet object"

LoadEffectOnSpeed::usage = "LoadEffectOnSpeed[f_:0, c_:.10] interpolates from the thought that empty ships may go 10% faster at the same fuel consumption. If the load factor is f, and c is the per unit possible increase then the load effect is (1 + c (1 - f)). Note that this has no effect on the fuel use per day, but on the distance or the hours of the sail"

MostProfitableSpeed::usage = "Keyword. Maximal profit per day, maximised over speed. Depends upon tariff, delays and port time"

MetacentricHeight::usage = "MetacentricHeight[roll period, b] gives the 
metacentric height for beam b and roll period. See Sea Transport p11. With 
point of gravity G (chosen along the center line of the ship) and point of 
Buoyancy B, then metacenter M is the point where the upward force of buoyancy cuts the center line of the ship"

NT::usage = "NT[Vc, D, d] gives the net tonnage, where Vc is the total volume of cargo spaces in cubic meters, d the Summer draft and D the depth of the ship\nNT[Vc, {dwt, D}] gives the same result while using Draft[dwt] as approximation\nNT[Vc, A:.6] for A the draft/Depth ratio"

Port::usage = "Keyword for harbour characteristics"

RollPeriod::usage = "RollPeriod[GM, b] gives the roll period for beam b and 
metacentric height GM. See Sea Transport p11. With point of gravity G and 
point of Buoyancy B, then metacenter M is the point where the upward force of buoyancy cuts the center line of the ship"

ShipCharge::usage = "If the density of the cargo is greater than 1 (Mass / Volume > 1) the charge is based on the weight of the cargo. In that case the deadweight tonnage is the limiting factor for the ship's capacity. Otherwise, when the density is less than 1, the charge is based on the volume of the cargo. This volume times the standard density of 1 Tonne / Meter^3 again gives a weight category, called a 'measurement ton'. In that case the NT is the limiting factor for the ship's capacity. \nShipCharge[Mass, m, d] or ShipCharge[Volume, v, d] for mass m, volume v and density d = m / v"


ShipCost::usage = "ShipCost[dwt, distance] gives the $/ton as a function of ship size in dwt and the one-way sailing distance in nautical miles"

ShipDraft::usage = "ShipDraft[dwt] gives the average length of a ship in meters as a function of ship size in dwt"

ShipLength::usage = "ShipLength[dwt] gives the average length of a ship in meters as a function of ship size in dwt"

ShipWidth::usage = "ShipWidth[dwt] gives the average length of a ship in meters as a function of ship size in dwt"

ShipPlot::usage = "ShipPlot[\"Cost\"] plots the contours of ShipCost[dwt, 
distance] for the defined distances in the package\nShipPlot[\"Draft&Width\"] plots the contours of ShipDraft and ShipWidth\nShipPlot[\"Length\"] plots the contours of ShipLength.\nThe latter two plots print with a spread"

ShippingModel::usage = "ShippingModel[{x___},y___List, opts___Rule] is a SolveFrom application. Do SetOptions[ShippingModel, Equations -> the list of equations that one wishes to use], default are $TimeCharterEquations"

SlendernessRatio::usage = "SlendernessRatio[l, d] gives l / d^(1/3) for length l and displacement d. Ships with speeds over 30 knots have slenderness ratios of about 7. A slenderness larger than 12 means insufficient stability"

TimeSheet::usage = "Keyword for a Databank application of LayTime, CraneTime, Demurrage and Despatch accounting"

$TimeCharter::usage = "Keyword.\nEquations[$TimeCharter] give the list of equations for the TimeCharterEquivalent, i.e. to transform the revenue of a single trip for a particular duration into a trip-independent $/day value. Also defined are:\nListOfSymbols[$TimeCharter] are short symbols used in ToShippingSymbols. These short symbols have been protected.\nParameter[$TimeCharter] is an example list of values"

ToShippingSymbols::usage = "ToShippingSymbols is a list of rules to translate long symbols to short symbols"

TPI::usage = "TPI[dwt] gives the tons per inch immersion per deadweight dwt in tons. TPI is the number of tons that must be loaded or discharged, to alter the draft one inch. Note: TPC = tonnes per centimeter = .4 TPI"


ListOfSymbols[$TimeCharter] = (*here to directly declare*)

{{DWT, Tonne, Deadweight, "Note: Tonne and not Ton"},
{R, Dollar/Tonne, SpotFreightRate, ""},
{L, Rate, LoadFactor, "Here: Load / Deadweight"},
{P, Dollar, Price["PortCharges"], "Per trip"},
{Pf, Dollar/Tonne, Price["HeavyMarineFuel"], ""},
{Pd, Dollar/Tonne, Price["Diesel"], ""},
{H, Day, Days[Handling], "For loading and unloading"},
{W, Day, Days["Wait"], ""},
{M, NauticalMile, Distance[], "Note that M can be 0"},
{MB, NauticalMile, Distance[Empty], "Ballast leg"},
{MT, NauticalMile, Distance[Sum], "Total distance"},
{TM, Tonne NauticalMile, Production, "Tonne Miles"},
{B,  Rate, BallastFactor, MB/M},
{cH, Tonne/Day, Consumption["Diesel", Handling], ""},
{cW, Tonne/Day, Consumption["Diesel", "Wait"], ""},
{cL, Tonne/Day, Consumption["Diesel", Lift], ""},
{S, NauticalMile/Hour, Speed, ""},
{BSF, Rate, Speed[Empty,Factor], "E.g. 10% faster but same fuel use"},
{DMB  , Day,  Days[Empty], ""},
{DM  , Day,  Days[Load] , ""},
{DL  , Day,  Days[Lift] , ""},
{DT  , Day,  Days[Sum] , ""},
{V, Dollar, Revenue["Voyage", "Net"], ""},
{$pTM  , Dollar/(Tonne NauticalMile),  Rate["Voyage", "TM"], V / TM},
{T, Dollar/Day, TimeCharterEquivalent, "Thus, equivalent for R"},
{qV, Dollar/Tonne, Rate["Voyage", "DWT"], V / DWT},
{N, Number/Month, Frequency, "days per month / days per trip"},
{qT, Dollar/Tonne/Month, Rate["TimeCharterEquivalent"], qV N}
} // Sort

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
