(*********************************************************************

	Cool`Statistics`Common`  package


 *********************************************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1996 Thomas Cool *)
(* :Package Version: 2.09 Thomas Cool, July 1998*)
(* :Mathematica Version: 3.0.0 *)
(* :Keywords:    *)
(* :Limitation:   *)
(* :License: use of this software file requires a license usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of packs. See the License[] function. *)

Needs["Cool`Common`"]

BeginPackage["Cool`Statistics`Common`", 
Join[{"Statistics`DescriptiveStatistics`", "Statistics`DataManipulation`", "Statistics`NormalDistribution`", 
"Statistics`DiscreteDistributions`",
"Statistics`Common`DistributionsCommon`",
"Statistics`Common`PopulationsCommon`"}, $Cool]]

AddedUsage[{Quotient, Normal, N, Position, Range, Context, Factor, Point}]


Accept::usage = "Keyword. For example, a lot is accepted (a) when first two draws are good, (b) one of these two is bad, but a third draw is good. Then:\n   Pr[Accept] = Pr[g1 && g2] + Pr[g1 && !g2 && g3] + Pr[!g1 && g2 && g3]"

Reject::usage = "Keyword"


Act::usage = "Head of actions"
Est::usage = "Head of estimates"
Obs::usage = "Head of observations"
Dec::usage = "Head of decisions"
State::usage = "Head of states"
Pr::usage =  "Keyword for probability"
Error::usage = "Keyword for the error term"


Categories::usage = "Keyword: the categories per dimension."

BarChartServer::usage = "bcs = BarChartServer[data_List, stepfactor_:Automatic] for flat data produces {Factor -> stepfactor, Range -> {Min, Max, step}, BinCounts -> freq, Point -> midpoints}, for BarChart[Transpose[{BinCounts, Point} /. bcs]]. The automatic step factor .1"

CNormalInverse::usage = "CNormalInverse[level, {mu, sigma}] gives the value on the x-axis for the cumulative normal = level"

Correlation::usage = "Correlation[y_List, x_List] gives the centered correlation coefficient, covar[y-my, x-mx] / Sqrt[var[y-my] var[x-mx]]"

DivSmallestNonZero::usage = "DivSmallestNonZero[x_] returns the value of x divided by its smallest nonzero element."

DegreesOfFreedom::usage = "Keyword: the degrees of freedom in statistical testing. E.g. DegreesOfFreedom -> Integer value."

FrequencyCategories::usage = "FrequencyCategories[freq_List]\n takes the output of Statistics`DataManipulation`Frequency[] as input, and selects the categories in that list. It would equal Union[data], but sometimes the data are not present."

FrequencyDispersion::usage = "FrequencyDispersion[x_List] if x has the frequency format of the Frequencies[] routine in the standard Mathematica statistics packages {{f1, category/state1}, {f2, category2}...}.\nFrequencyDispersion[ListPlot, x_List] if x has the frequency format of ListPlot {{category1, f1}, {category2, f2}...}.\nGives a dispersion report with the statistics weighted by the frequencies. The result can be input of NormalProxy to give a proxy by the Normal distribution using the mean and standard deviation."

FrequencyValues::usage = "FrequencyValues[freq_List]\n takes the output of Statistics`DataManipulation`Frequency[] as input, and selects the frequency values in that list."

PrTable::usage = "PrTable[t,p, ...] or PrTable[{t,p, ...}] = Outer[Times, t, p, ...] \ngives the probability of occurrence of cells, when the states have distribution t and the actions or decisions have distribution p, etc."

ID::usage = "A keyword for denoting independent distributions."

Options[MissingToZero] = {MessagesQ -> False, Context -> "Global`"}

MissingToZero::usage = "MissingToZero[var_, categories_List, opts___Rule]\nStatistics`DataManipulation`Frequencies[] gives data of the form {... {freqi, categoryi} ...}, and here {0, categoryj} is added for missing categories.\nIf var is a String, then it is taken as the variable in the context specified by the Context -> option (default Global`). MessagesQ -> False, default, suppresses shadowing messages, by using ShadowHull."

MultinomialDistribution::usage = "PDF[MultinomialDistribution[p_List, n, opts], x_List] gives the probability density of the Multinomial Distribution. Default: p and x are specified for k-1 variables, and the routine adds the kth term, using PM[Join[p, {Rest}]] and making xk = n - Plus @@ x. If option Add -> False, then this enlargement is not done. Note: the function is defined for xi = 0, 1, ..., n. CDF[MultinomialDistribution[p_List, n_, opts___Rule], c_] gives the cumulation up to and including c"

NormalProxy::usage = "NormalProxy[x_Symbol, d_List, dcf_:Automatic] uses a list with dispersion statistics to construct a normal distribution, using variable x, that approximates the original data. Output options are:\nPDF -> pdf, gives the normal distribution, such that NIntegrate[pdf, {x, -Infinity, Infinity}] returns the NumberOfObservations;\ndcf is used as a scalar discplay correction factor, to allow a better comparison of the plot of the PDF with the ListPlot of the original data. If dcf = Automatic, then the dcf is computes as SampleRange / NumberOfStates; output also gives Factor -> computed dcf;\nPlot -> h, gives a plot in Hold form, with the display correction factor and for the given sample range. Use ReleaseHold[ Plot /. output] to actually generate the plot" 

LogNormalToNormal::usage = "LogNormalToNormal[theta_, tau_] takes the 'mean' and 'sigma' parameters of the LogNormal distribution, and translates these in the proper mean and standarddeviation. Also the median is given (as Exp[theta]) and the ratio between mean and median. Normally, the theta differs from Log[mean] by a certain amount"

NormalToLogNormal::usage = "NormalToLogNormal[mean_, stdev_, median_:Automatic] takes sample observations mean and stdev, and generates the parameters of the associated LogNormal[theta, tau] distribution. If the sample median is supplied, then Log[median] is given, and its difference with theta. Log[median] ideally is an estimator of theta, but of course the sample data may not fit the LogNormal distribution"


NumberOfStates::usage = "Keyword that may occur in a rule like 'NumberOfStates -> Number' that gives the number of states of nature. Sometimes: the number of categories."

NumberOfActions::usage = "Keyword that may occur in a rule like 'NumberOfActions -> Number' that gives the number of possible actions."

NumberOfDecisions::usage = "Follows as NumberOfActions^NumberOfStates."

NumberOfObservations::usage = "Keyword that may occur in a rule that gives the number of observations."

Outliers::usage = "Outliers[k_, delta_List, mu_:0] gives the outliers for k times the standard deviation:  Abs[delta-mu] > k sigma. Standard is delta = x-mean[x], so that mu = 0. Output is {Range -> k, N -> number of outliers, Quotient -> rate of outliers, StandardDeviation -> sigma, Normal -> normalized data (delta-mu)/sigma, Position -> positions of the outliers}. Then use TakeElements."

PM::usage = "PM[x_List] makes a probability measure of x\n'Rest' in the list is set to 1 - sum[x]\nNot is applied when ProbabilityMeasureQ fails."

PML::usage = "PML[t_] := If[ListQ[t], PM[t], PM[{t, Rest}] ]"

PowerOfTheTest::usage = "PowerOfTheTest[a, N -> n] gives a probability matrix when the PowerOfTheTest of the test (rate of failure of estimation) is a. Alternatively put, the quality of the test then is 1 - a = Pr[Est[i]; State[i]], {i,1,n}. Out-values are even-spreaded a/(n-1)).\nPowerOfTheTest[a] has default N -> 2.\nPowerOfTheTest[a,b] gives {{1-a, a},{b,1-b}}"

PrMarginal::usage = "Keyword for a rule giving the marginal probabilities."


PrBelowOutMean::usage = "PrBelowOutMean[x_Symbol, m_Symbol] is measure of 
skewness. It tells how far apart the two conditional means E[x | x > m] and E[x | x < m] are from the mean m of a distribution. The measure is simply the area in the density function on the left of the mean, adjusted for the area taken in by the mean itself. For continuous densities Pr[x == m] = 0, so there is no adjustment. For discrete densities the adjustment is (1 - Pr[x == m]). This measure is simpler than Tajuddin and basically gives the same.\nPrBelowOutMean[list] gives the measure for a list of data. \nPrBelowOutMean[freq_List, values_List] gives the measure using frequency data.\nEven with discrete data, the probability Pr[x == m exactly] is often null. So you may want to round the estimated mean m, e.g. to the nearest integer. Use the option RoundAt -> n for the number of digits n. Default RoundAt -> False.\nResults[PrBelowOutMean] contain the mean, the Pr[x < m] and the Pr[x == m]"

ProbabilityMatrix::usage = "Keyword. In a ProbabilityMatrix -> {numericvalues} option, it depends upon the routines whether the numeric values per row must add up to one, or that such holds for the whole matrix."

ProbabilityMeasureQ::usage = "ProbabilityMeasureQ[x_List] tests whether x is a probability measure, i.e. x[i] >= 0 and sum[x] = 1"

SubsequentBetaCDF::usage = "SubsequentBetaCDF[lis_List, t, printq:True] gives the distribution of the cumulative duration t of a list of subsequent Beta events or activities. The activities are scored on the minimal, mode, and maximal time, so that lis = {..., {mini, modei, maxi}, ...}. The function ToBeta then generates estimates of means and variances, and these can be summed for the total. Output is the Normal CDF with this total mean and standarddeviation, which are printed if printq === True. Variable t can be symbolic or have a numeric value"

Tajuddin::usage = "Tajuddin[...] gives a measure of skewness. It tells how far apart the two conditional means E[x | x > m] and E[x | x < m] are from the mean m of a distribution. The measure is simply the logarithm of the ratio of the areas in the density function on the left and the right of the mean. For continuous densities Pr[x = m] = 0, so the '<' and '>' are without problem. For discrete densities exclude Pr[x = m], so set: denominator = ((1 - Pr[x = m]) - numerator). See: Statistica Neerlandica 1996, pp 362-366.\nTajuddin[x] is a transform of PrBelowOutMean[x], and see the input rules there"

Test::usage = "Keyword for a rule, showing the kind of test applied."

TestStatistic::usage = "Keyword for a rule, showing the value of the test statistic according to the Test -> (kind of test applied)."

ToFrequencyDensity::usage = "ToFrequencyDensity[pairs_List] reworks a list of frequency pairs {{f1, cat1}, {f2, cat2} ...}. It is assumed that fj observations fall in the class of cat(j-1) till catj, so that the mean position is (catj + cat(j-1))/2, while the frequency must be adjusted for the width of the range to get the frequency density. Necessarily f1 falls out, the first pair is mainly useful for cat1, and the output is one pair shorter than the input. Output is {Frequencies -> {list of pairs with adjusted means}, ListPlot -> {list of pairs with adjusted means and frequency densities, reversed for ListPlot}}. The first would be used to determine sample statistics, the second would be used for ListPlot"

TValues::usage = "Keyword for Student t values."

ToBeta::usage = "ToBeta[min, mode, max] gives estimates of the mean and variance of the Beta distribution, using minimal, most likely and maximal values\nToBeta[min, mode, max, All] gives {{mu, var}, {alpha, beta}}"

VariancePr::usage = "VariancePr[x_List, p_List] gives the variance of a list of values x using the list of associated probabilities p"

WeightUnit::usage = "When the Weight is not Null, then the weights may be adjusted by using WeightUnit and WeightSet. Conventions are: When WeightUnit -> False, no adjustment takes place. When WeightUnit -> f, then adjusted weights are w = f[w]. An example is f = (# / Add[#])&. Another example is f = DivSmallestNonZero, which divides the weights by the smallest nonzero element (that might be negative). Note that some unit adjustment may be needed, since 'observations' or 'cases' are natural numbers, while weights may be inflated to any level and thereby affect results."

WeightTotal::usage = "Keyword for the total value of the weights"

Weight::usage = "Keyword for indicating weights. Default would be Null. See WeightSet and WeightUnit."

WeightSet::usage = "WeightSet[rule_] gives proper weights from the rule. It uses Weight (default Null) and adjusts with WeightUnit (default False)."

WeightedMean::usage = "WeightedMean[c_List, fr_List] weighs c by using the frequencies fr. The weighing function WeightUnit is used from Options[WeightedMean], with default fr/Add[fr].\nWeightedMean[x_List] uses the output x = Frequencies[...] = {fr, c} (reversed)\nSee Results[WeightedMean]" 

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
