(*******************************************

	Cool`TScience`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1997 Thomas Cool *)
(* :Package Version: 1.03,  July 1998.*)
(* :Mathematica Version: 3.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function.
 *)

Needs["Cool`Common`"]

BeginPackage["Cool`TScience`", Join[{"Miscellaneous`SIUnits`","Miscellaneous`Units`", "Cool`Arrowise`", "Cool`Physics`"}, $Cool]]

AirplanePropulsion::usage = "AirplanePropulsion[ACW, LDR, g:9.8] gives the 
propulsion in kilo Newton, for average cruising weight (ACW) in tonnes, LDR 
the the ratio between air lift and air drag (resistance) and the 
gravitational acceleration g";

MachToKPH::usage = "MachToKPH[x] converts Mach x into kilometers per hour";

BackSideOfTruckGP::usage = "BackSideOfTruckGP[{w_, h_, a_}, {ww_, hw_, b_}, 
c_List:{0,0}] gives the graphics primitive of the backside view of a truck. 
The truck has width w, height h, angle a with the horizontal axis; while the tyres are of the size width ww and height hw, and are located at an angle b with the normal of a\nExample: Show[Graphics[BackSideOfTruckGP[{4, 4, -Pi/10}, {.4, 1, Pi/12}]], AspectRatio -> Automatic]"

BrakingModel::usage = "BrakingModel[{x__}, y___List, opts___Rule] is a SolveFrom application, using the model:
\n      Distance == 1/2 Speed^2 / Acceleration
\n      Acceleration == Gravity Friction
\n      Duration == Speed / Acceleration"

BunkerGP::usage = "BunkerGP[x_:0, y_:0, d_:1] is a graphics primitive for a 
picture of a bunker at point {x, y} with dimension d. Subroutine of 
GrabBunkerBeltShow"

DiggingPhases::usage = "DiggingPhases[] plots the digging phases of a grab and a continuous unloading machine"

GrabGP::usage = "GrabGP[t_List:{0,0}] is a graphics primitive for a picture of a grab at point t. Subroutine of GrabBunkerBeltShow"

BeltGP::usage = "BeltGP[{x_:0, y_:0}, r_:1, len_:10] is a graphics primitive for a picture of a belt at point {x,y} with roller radius r and length len. Subroutine of GrabBunkerBeltShow"

InFlow::usage = "Keyword for the inflow in a bunker";

OutFlow::usage = "Keyword for the absolute value of the outflow in a bunker";

NetInFlow::usage = "Keyword for OutFlow - InFlow";

BunkerModel::usage = "BunkerModel[{x__}, y___List, opts___Rule] is a SolveFrom application, using the model:
\n      OutFlow == Volume[\"Out\"] / Duration[\"Out\"]   for the process 
served
\n      NetInFlow == OutFlow - InFlow
\n      NetInFlow == Volume[\"Bunker\"] / Duration[\"Out\"]
\n      InFlow == Volume[\"Bunker\"] / Duration[\"In\"]
\n      CycleTime == Duration[\"Out\"] + Duration[\"In\"]"

EnergyEfficiency::usage = "EnergyEfficiency[p, v, f, SFC, HV] gives the energy efficiency of a vehicle.
\nLegend:\np = payload (Tonne)\nv = speed (Kilo Meter / Hour)\nf = propulsion (Kilo Newton), \nSFC = specific fuel consumption, or the use of fuel per unit propulsion force per hour (Kilogram / (Newton Hour))\nHV = heat value, or the energy released by burning a kilogram of the fuel (Kilo Joule / Kilogram)
\n
\nOutput is:
\nFuelConsumption -> fuel use (Tonne / Hour)
\nTransport -> the transport result or output (Tonne Kilo Meter / Hour)
\nEnergyConsumption -> the flow of energy (million Joule / Hour)
\nSpecificEnergyConsumption -> the use of energy (million Joule / (Tonne Kilo Meter))
\nEnergyEfficiency -> a dimensionless number
\n
\nExample:
\nEnergyEfficiency[55, MachToKPH[.8], AirplanePropulsion[310, 15], 0.062 , 
43000 ]
";

GantryCraneGP::usage = "GantryCraneGP[{p1, h1}, {p2, h2}, {w1, w2}] gives a graphics primitive of a gantry crane. The first pole is at {p1,0} and has a height of h1. The second pole is at {p2,0} and has a heigth of h2. The distance of the first pole to the water is w1, and the cat length is w2"
	
ShipGP::usage = "ShipGP[w, h, a, c_List:{0,0}] gives a graphics primitive of a front or back view of a ship, with width w, height h, angle a, and centered at c. It is supposed that the hatch is 2/3w and h/10. See BlockGP"

GantryShipShow::usage = "GantryShipShow[x:68, angle:0] shows a typical combination of a gantry crane and a ship, where the grab is at position x on the cat, and the ship makes an angle with the water."

GrabBunkerBeltShow::usage = "GrabBunkerBeltShow[t_List:{0,0}] shows a Grab, 
Bunker and Belt at point t"

GrabBunkerBeltPlot::usage = "GrabBunkerBeltPlot[m:9000, c:45, ot:3, time:120, opts] plots the performance of this system for mass m, cycle period c, grab unloading time ot, and for the range of {0, time}. Using only the defaults, also labels m(in) and m(out) are included. Opts are passed on to Plot. Note: you may have to increase the PlotPoints"

GrabLiftForce::usage = "GrabLiftForce[dwt] gives an approximation of the historical development of the lift force of crane grabs as a function of ship's dwt. GrabLiftForcePlot[] plots this for the range 30 - 300 dwt."

GrabLiftForcePlot::usage = "GrabLiftForcePlot[dwt] gives an approximation of the historical development of the lift force of crane grabs as a function of ship's dwt, for the range 30 - 300 dwt."

GrabVolumeFlow::usage = "GrabVolumeFlow[V, T, o] gives key results of grab volume V, grab open time T and final output flow o = dVdt(out) that is required for the process served.\n\nGrabVolumeFlow[TGW, k, rho, T] gives the (bunker input) volume flow of a grab with total mass TGW = PL + DW = payload + deadweight (kg), k = payload deadweight ratio = PL / DW, a material density of rho (kg / m3), and (grab open) time of T (seconds)\n\nGrabVolumeFlow[TGW, k, rho, T, outflow] gives the same result as above, but also the grab and bunker volumes\n\nGrabVolumeFlow[{___}] is an application of SolveFrom[], using the equations:\n1) VolumeFlow[Out] == Volume[\"Grab\"] / Duration[\"Cycle\"],\n2) VolumeFlow[In] == Volume[\"Grab\"] / Duration[\"GrabOpen\"],\n3) Volume[\"Bunker\"] == Volume[\"Grab\"] - VolumeFlow[Out] Duration[\"GrabOpen\"],\n4) Mass[Load] == Density Volume[\"Grab\"],\n5) PayloadDeadweightRatio == Mass[Load] / Mass[NWeight],\n6) Gross == Mass[Load] + Mass[NWeight]"

PipeElevatorCrosssectionPlot::usage = "PipeElevatorCrosssectionPlot[opts___Rule] only gives a plot of a spiral, that gives an impression of a section of a pipe elevator"

PipeElevator::usage = "PipeElevator[B, o, v] gives the volume flow of a pipe elevator for belt width B (in meters), an overlap rate o (10% = .1) and velocity v (meters per second). Another result is the diameter of the pipe."

PipeElevatorWidth::usage = "PipeElevatorWidth[dVc/dt, o, v] gives the belt width (in meters) for a volume flow dVc/dt, an overlap rate o (10% = .1) and velocity v (meters per second)"

SafetyMargin::usage = "Keyword"

SpecificFuelConsumption::usage = "Keyword for the kilograms fuel per unit 
propulsion force per hour (kg / (N h))"

SpecificEnergyConsumption::usage = "Keyword for the use of energy (million 
Joule / Tonne-Kilometer)"

TopplingCraneGP::usage = "TopplingCraneGP[] gives a graphics primitive of a 
toppling crane. Use e.g. as Show[Graphics[TopplingCraneGP[]], AspectRatio -> Automatic]"


TopplingCrane::usage = "TopplingCrane[x___Rule] is a SolveFrom application, 
using
\n    Load == Constant / Distance[\"Reach\"],
\n    Constant == NWeight Distance[\"Pivot\"] / (1+SafetyMargin),
\n    Distance[\"Reach\"] (1+SafetyMargin) == Distance[\"TheoreticalReach\"]"




TruckGP::usage = "TruckGP[d, h, r, p:{0,0}] gives a graphics primitive for a simple picture of a truck. The center of the back tyre is at p, with default p = {0,0}. The center of the front tyre is at distance d. The radius of the tyres is 1. The height of the truck is h, and r measures the radius of the window, i.e. the distance from the front to the back of the front tyre.\nUse as in Show[Graphics[TruckGP[8, 5.5, 3]], AspectRatio -> Automatic]"

VdVdtPlot::usage = "VdVdtPlot[V, vin, vout] gives the plot of V and dV/dt when a bunker with maximum content V first has volume flow v(in) and then volume flow -v(out). Possibly, -v(out) is the net of v(in) - v(truely out), when the inflow never stops while there is only temporarily a strong outflow"

WeighingBunkerShow::usage = "WeighingBunkerShow[] shows the construction of three bunkers used for weighing, when inflow and outflow are continuous"

PayloadDeadweightRatio::usage = "Keyword for GrabVolumeFlow"



Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
