(*******************************************

	Cool`Voting`  package


 *******************************************)



(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1997 Thomas Cool *)
(* :Package Version: 1.00,  June 1997.*)
(* :Mathematica Version: 3.0 *)
(* :Keywords:    *)
(* :Limitation: *)
(* :License: use of this software file requires a license, usually available
      by payment of $10 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function.
 *)

Needs["Cool`Common`"]

Print["There should be shadowing messages on Ordering and Star. These messages concern Mathematica packages over which the present author has no control"]

BeginPackage["Cool`Voting`", Join[{
"Cool`Statistics`Common`", "DiscreteMath`Combinatorica`", 
"DiscreteMath`Permutations`",
"Graphics`MultipleListPlot`"}, $Cool]]

StatusQuo::usage = "StatusQuo[] gives the status quo. By default the first of Items"

CheckVote::usage = "CheckVote[] gives a check on the voting data"

BordaField::usage = "BordaField[p_:Preferences, v_:Votes] simply applies the Votes to Preferences"

Borda::usage = "Borda[p_:Preferences, v_:Votes, i_:Items] chooses the items with the maximum in the BordaField"

BordaAnalysis::usage = "BordaAnalysis[p_:Preferences, v_:Votes, i_:Items] gives a general analysis of the situation for a Borda type of vote:\n   1) the selected items\n   2) the BordaField\n   3) the positions of the maxima\n   4) the items sorted from lowest to highest weighted vote"

DuncanBlackPlot::usage = "DuncanBlackPlot[x, plotting opts] plots the preferences with the items on the x-axis and the values on the y-axis. May be used to see single-peakedness. Default are the Preferences weighted by the Votes. For Preferences only, evaluate DuncanBlackPlot[Preferences]"

ExamplePrefs::usage = "ExamplePrefs[n] for n =1,2 give example Preferences for 3 voters and 3 items (with e.g. weighted voting)"

EfficiencyMajority::usage = "EfficiencyMajority[p, v] with default Preferences and Votes, first selects the EfficiencyPairs that dominate the Status Quo, and then applies the majority rule to those. Selected points thus always satisfy the Paretian criterion. However, if {A, B} and {A, C} are both Paretian improvements, but there is no clear Efficiency preference on {B, C}, then there might still be a deadlock. These points are given in output with the option PairsToPath -> {...}. The EfficiencyMajority rule solves this deadlock by majority voting, so that that Paretian improvement is selected which gets most votes.\nNote: A final deadlock of indifference by equal votes is left to the user; you may select the Status Quo, use dice, etc."

GroupOrderQ::usage = "GroupOrderQ[x, y] is set by PairwiseField[] and gives the implied preference order. The order is the same as LessEqual: the last position in the list is the most important"

Items::usage = "Items takes NumberOfItems elements of the Alphabet. Note that you can set your own Items = {...}"

Pairs::usage = "Pairs[i_:Items] gives the list of unordered pairs in i"

NPairs::usage = "NPairs[n_:NumberOfItems] gives the list of unordered pairs for the set {1, ..., n}"

Pairwise::usage = "Pairwise[preference list, pair_List] gives the element uit pair that has highest preference. It is assumed that pair is a list of two numbers, where the numbers give the positions of items in Items.\nPairwise[preference list, v_] assigns votes v to the position with highest preference.\nIn both cases the majority rule has been used, but you are free to give another implementation."
	
PairwiseVoter::usage = "PairwiseVoter[n_Integer, pair_List] performs the pairwise vote for voter n using his preferences and voting power. It is assumed that pair is a list of two numbers, where the numbers give the positions of items in Items."

PairwiseField::usage = "PairwiseField[list of pairs] applies PairwiseVoter to all voters, for that list of pairs. Numerical pairs must be used. For output: see PairwiseMajority."

PairwiseMajority::usage = "PairwiseMajority[p, v] with default Preferences and Votes, takes the PairwiseField[] of NPairs[].\nOutput is:\n   1) the matrix of voters and the pairs under consideration\n   2) the matrix of votes cast per voter and pair\n   3) the total vote per pair (column sums)\n   4) the PrefPairs object\n   5) the selected item per pair (using Pairwise[] that gives majority voting)\n   6) the maximal items sorted by the group preferences in PrefPairs\n   7) the last element of the sorted list (possibly a list if indifference is found)\n   8) the selected element (possibly the Status Quo)"

Preferences::usage = "Preferences is a {NumberOfVoters, NumberOfItems} matrix (list of lists) for the values assigned to the items, in the order of Items. A higher value means a higher priority. Thus {{1, 2}, {1, 2}} means that there are two voters that both assign a higher value to B rather than A"

Pref::usage = "Pref[x1, ..., xn] gives a preference order from the lowest preferred x1 to the highest preferred xn. The position in the order in fact gives the ordinal preference value. Elements of equal preference are put in sublists, such as Pref[x1, {x2, x3}]. Note that the Pref[..] object has not been taken as the basic object. The reason is that sorting gives less information, and the size of the gap between the various alternatives can best be put into numbers. However, for majority voting, the Pref[..] object does good service to help eliminate cycles in the final selection"

PreferencesPlot::usage = "PreferencesPlot[x, plotting opts] gives a density plot of x, with default x = Preferences. Note: highest - > white shading, lowest -> black shading" 

NumberOfItems::usage = "Must be set to the number of Items considered"

NumberOfVoters::usage = "Must be set to the number of voters considered"

NPrefOrderQ::usage = "NPrefOrderQ[pair_List, preference_List] can be used for Sort, and gives False if the first element in pair comes before the last, according to the stated preference list. Pair is assumed to consist of numbers"

PrefOrderQ::usage = "PrefOrderQ is a head only. See ListToPrefOrderQ. If an order is defined, then it can be used for Sort. It gives False if the first element in a pair comes before the last, according to the stated preference list. A pair is assumed to consist of elements of Items"

	
ListToPref::usage = "ListToPref[preference list (,oq)] sorts Items on the preferences, to created a sorted Pref[...] object. You may optionally enter a head name oq, default is PrefOrderQ.\nListToPref[\"Items\", it, oq] sorts on items it. Here oq must be known."

ListToGraph::usage = "ListToGraph[x_List, opts___Rule] performs  PrefPairsToGraph[ListToPrefPairs[x], opts]"

SetRandomPrefPairs::usage = "SetRandomPrefPairs[n_:NumberOfItems, type_:Integer, ran_List:{-1, 1}] creates a n x n matrix of Random[type, ran] values, though with diagonal 0. The Head PrefPairs is added, to distinguish this matrix from the normal preference ordering that is represented by a List"

NOrderPair::usage = "NOrderPair[preference list, {i, j}] makes the ordered pair for numbered items i and j, based on the preference list for one voter. If there is indifference between item1 and item2, both {item1, item2} and {item2, item1} occur. There is a strict preference if the opposite pair does not occur."

ListToOrderedPairs::usage = "ListToOrderedPairs[preference list, items] changes the preferences list (for one voter) into a list of ordered pairs. See NOrderPair"
	
EfficiencyPairs::usage = "EfficiencyPairs[] gives the ordered pairs in Preferences that satisfy the Efficiency criterion, i.e. those pairs {item1, item2} where someone improves by taking item2 instead of item1, while nobody is hurt.\nOther formats are EfficiencyPairs[{preference list1, preference list2, ...}] and EfficiencyPairs[preference list1, preference list2, ...] where the first format is computationally more efficient. Note: you may want to use GraphIntersection[]"

PrefPairsPlot::usage = "PrefPairsPlot[pp_PrefPairs, opts___Rule] gives a density plot of a PrefPairs object"

PrefPairs::usage = "PrefPairs[{row1, row2, ..., rown}] is a separate object to represent the square matrix of outcomes of pairwise comparisons of n items. Thus each element PrefPairs[i, j] is the outcome of a preference consideration. Assumed is that 0 means Indifference, and it applies to the diagonal (in the plot from bottom left to top right). A positive value means that i is less than j, a negative value conversely. The size of the value may matter, depending upon the application. If PrefPairs[i, j] + PrefPairs[j, i] =!= 0, then the preference pairs are 'irrational', as sometimes happens in experiments"

ListToPrefPairs::usage = "ListToPrefPairs[preference_List] changes a preference list into a PrefPairs[...] object, by assuming that the utility difference between the items is simply the difference of the values in the preference list"

PrefPairsToOrderQ::usage = "PrefPairsToOrderQ[pref_PrefPairs, oq] translates the PrefPairs object into a sorting order criterion oq, that can be used for Sort. For example oq = PrefOrderQ[i] for the ith voter"

PrefPairsToGraph::usage = "PrefPairsToGraph turns a PrefPairs object into a Graph, which may provide a bridge to the routines in DiscreteMath`Combinatorica`, such as ShowGraph. Note that indifference can be represented at least in two ways in graphs:\nPrefPairsToGraph[x_PrefPairs, SameQ -> True]\n   represents indifference by a value 0 in the adjacency matrix. In that case the adjacency values represent the 'distances' between points, so two indifferent items are plotted at the same vertex.\nPrefPairsToGraph[x_PrefPairs, SameQ -> False] (default)\n   sets all off-diagonal nonnegative values to 1, including zero values, so that all items can be plotted separately, and indifference is given by two arrows 'to and fro' points.\nBoth results would be plotted with ShowGraph[graph, Directed]"

PairsToPaths::usage = "PairsToPaths[pairs_List] chains pairs to form paths. Output are the pairs that form beginning and end of the longest paths, and with the intermediate pairs thus eliminated.\nNote: cycles are not looked for. The result is stored in PPath. Evaluate ??PPath"

PPath::usage = "PPath[{a, b}] gives the list of pairs that together form a path from a to b. See PairsToPaths"

Condorcet::usage = "Condorcet[] sets key parameters to the example voting paradox given by Marquis de Condorcet 1785"

SetRandomPreferences::usage = "SetRandomPreferences[] sets the Preferences to random orderings; or\nSetRandomPreferences[n_] sets the number of items to n, and then generates random preferences"

ShowListGraph::usage = "ShowListGraph[x_List, opts] performs ListToPrefPairs, PrefPairsToGraph, and ShowPrefGraph. The options apply to these and Show. For two elements you may want to adjust the PlotRange"

ShowPrefGraph::usage = "ShowPrefGraph[graph, opts] shows a graph in a typical preference analysis situation: as a labeled directed graph. Adjust with options"

ScoreBoard::usage = "ScoreBoard[pairs, votes] takes a list of pairs and pairwise vote results, applies ScoreVote and generates a PrefPairs object. The pairs must be numerical, though"

ScoreVote::usage = "ScoreVote[{p, 1-p}] assigns a score to a pairwise vote where p votes go to the first item and (1-p) votes go to the other. We want to see a positive value if the first is less to the second, so the score = (1 - p) - p has been implemented here"

Votes::usage = "Votes gives the list of votes per voter. The sum must add to unity. The default for 3 voters is PM[{.25, .35, Rest}]"


VotingQ::usage = "The principle of the Voting package is as follows. The major objects are preferences, represented by lists. A preference list of a single voter gives a row v of values, where v[[i]] gives the value attached to the ith item, with a greater number for a greater value. The Preferences matrix has NumberOfVoters rows for voters and NumberOfItems columns of preference values. These preferences are transformed by Borda[], EfficiencyMajority[] or PairwiseMajority[] to generate Group results. DuncanBlackPlot[] plots the preferences for all voters.\nA preference list (for a single voter, or for the Group result) can be transformed into a PrefPairs[] object that gives another representation of the results of pairwise comparisons. The PrefPairs[] representation allows for cycles like A > B > C > A. Importantly, since preferences represented by lists cannot produce cycles, the PrefPairs[] representation is (primarily) relevant for the Group result. PrefPairsToGraph[] allows a translation to the existing Graph functions."


ListToPrefOrderQ::usage = "ListToPrefOrderQ[preference List, h_:PrefOrderQ] uses the preference list to define the h sorting criterion, with default PrefOrderQ. Then evaluate Sort[Items, h] for example. See ListToPrefPairs and PerPairsToOrderQ"

Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
